@@ -18,7 +18,7 @@
 
 int hcoll_comm_attr_keyval;
 int hcoll_type_attr_keyval;
-
+mca_coll_hcoll_dtype_t zero_dte_mapping;
 /*
  * Initial query function that is invoked during MPI_INIT, allowing
  * this module to indicate what level of thread support it provides.
@@ -153,9 +153,9 @@ static void mca_coll_hcoll_module_destruct(mca_coll_hcoll_module_t *hcoll_module
 }
 
 #define HCOL_SAVE_PREV_COLL_API(__api) do {\
-    hcoll_module->previous_ ## __api            = comm->c_coll.coll_ ## __api;\
-    hcoll_module->previous_ ## __api ## _module = comm->c_coll.coll_ ## __api ## _module;\
-    if (!comm->c_coll.coll_ ## __api || !comm->c_coll.coll_ ## __api ## _module) {\
+    hcoll_module->previous_ ## __api            = comm->c_coll->coll_ ## __api;\
+    hcoll_module->previous_ ## __api ## _module = comm->c_coll->coll_ ## __api ## _module;\
+    if (!comm->c_coll->coll_ ## __api || !comm->c_coll->coll_ ## __api ## _module) {\
         return OMPI_ERROR;\
     }\
     OBJ_RETAIN(hcoll_module->previous_ ## __api ## _module);\
@@ -204,7 +204,7 @@ static int mca_coll_hcoll_save_coll_handlers(mca_coll_hcoll_module_t *hcoll_modu
 /*
 ** Communicator free callback
 */
-int hcoll_comm_attr_del_fn(MPI_Comm comm, int keyval, void *attr_val, void *extra)
+static int hcoll_comm_attr_del_fn(MPI_Comm comm, int keyval, void *attr_val, void *extra)
 {
 
     mca_coll_hcoll_module_t *hcoll_module;
@@ -240,16 +240,6 @@ static int mca_coll_hcoll_module_enable(mca_coll_base_module_t *module,
     return OMPI_SUCCESS;
 }
 
-int mca_coll_hcoll_progress(void)
-{
-    if (ompi_mpi_state >= OMPI_MPI_STATE_FINALIZE_STARTED) {
-        hcoll_rte_p2p_disabled_notify();
-    }
-
-    (*hcoll_progress_fn)();
-    return OMPI_SUCCESS;
-}
-
 
 OBJ_CLASS_INSTANCE(mca_coll_hcoll_dtype_t,
                    opal_free_list_item_t,
@@ -290,7 +280,7 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
            mxm bcol in libhcoll needs world_group fully functional during init
            world_group, i.e. ompi_comm_world, is not ready at hcoll component open
            call */
-        opal_progress_register(mca_coll_hcoll_progress);
+        opal_progress_register(hcoll_progress_fn);
 
         HCOL_VERBOSE(10,"Calling hcoll_init();");
 #if HCOLL_API >= HCOLL_VERSION(3,2)
@@ -307,7 +297,7 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
 
         if (HCOLL_SUCCESS != rc){
             cm->hcoll_enable = 0;
-            opal_progress_unregister(mca_coll_hcoll_progress);
+            opal_progress_unregister(hcoll_progress_fn);
             HCOL_ERROR("Hcol library init failed");
             return NULL;
         }
@@ -328,19 +318,20 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
         if (OMPI_SUCCESS != err) {
             cm->hcoll_enable = 0;
             hcoll_finalize();
-            opal_progress_unregister(mca_coll_hcoll_progress);
+            opal_progress_unregister(hcoll_progress_fn);
             HCOL_ERROR("Hcol comm keyval create failed");
             return NULL;
         }
 
         if (mca_coll_hcoll_component.derived_types_support_enabled) {
+            zero_dte_mapping.type = DTE_ZERO;
             copy_fn.attr_datatype_copy_fn = (MPI_Type_internal_copy_attr_function *) MPI_TYPE_NULL_COPY_FN;
             del_fn.attr_datatype_delete_fn = hcoll_type_attr_del_fn;
             err = ompi_attr_create_keyval(TYPE_ATTR, copy_fn, del_fn, &hcoll_type_attr_keyval, NULL ,0, NULL);
             if (OMPI_SUCCESS != err) {
                 cm->hcoll_enable = 0;
                 hcoll_finalize();
-                opal_progress_unregister(mca_coll_hcoll_progress);
+                opal_progress_unregister(hcoll_progress_fn);
                 HCOL_ERROR("Hcol type keyval create failed");
                 return NULL;
             }
@@ -357,7 +348,7 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
         if (!cm->libhcoll_initialized) {
             cm->hcoll_enable = 0;
             hcoll_finalize();
-            opal_progress_unregister(mca_coll_hcoll_progress);
+            opal_progress_unregister(hcoll_progress_fn);
         }
         return NULL;
     }
@@ -376,7 +367,7 @@ mca_coll_hcoll_comm_query(struct ompi_communicator_t *comm, int *priority)
         if (!cm->libhcoll_initialized) {
             cm->hcoll_enable = 0;
             hcoll_finalize();
-            opal_progress_unregister(mca_coll_hcoll_progress);
+            opal_progress_unregister(hcoll_progress_fn);
         }
         return NULL;
     }
