@@ -1,21 +1,24 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2011 University of Houston. All rights reserved.
- * Copyright (c) 2014      Research Organization for Information Science
+ * Copyright (c) 2014-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -25,20 +28,20 @@
 #include "mpi.h"
 #include "ompi/constants.h"
 #include "ompi/mca/fcoll/fcoll.h"
-#include "ompi/mca/io/ompio/io_ompio.h"
+#include "ompi/mca/common/ompio/common_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "opal/mca/base/base.h"
 #include "math.h"
 #include "ompi/mca/pml/pml.h"
 #include <unistd.h>
 
-/*Based on ROMIO's domain partitioning implementaion 
+/*Based on ROMIO's domain partitioning implementaion
 Series of functions implementations for two-phase implementation
 Functions to support Domain partitioning and aggregator
 selection for two_phase .
 This is commom to both two_phase_read and write. */
 
-int mca_fcoll_two_phase_domain_partition (mca_io_ompio_file_t *fh,
+int mca_fcoll_two_phase_domain_partition (ompio_file_t *fh,
 					  OMPI_MPI_OFFSET_TYPE *start_offsets,
 					  OMPI_MPI_OFFSET_TYPE *end_offsets,
 					  OMPI_MPI_OFFSET_TYPE *min_st_offset_ptr,
@@ -48,7 +51,7 @@ int mca_fcoll_two_phase_domain_partition (mca_io_ompio_file_t *fh,
 					  OMPI_MPI_OFFSET_TYPE *fd_size_ptr,
 					  int striping_unit,
 					  int nprocs_for_coll){
-        
+
     OMPI_MPI_OFFSET_TYPE min_st_offset, max_end_offset, *fd_start=NULL, *fd_end=NULL, fd_size;
     int i;
 
@@ -58,57 +61,57 @@ int mca_fcoll_two_phase_domain_partition (mca_io_ompio_file_t *fh,
     for (i=0; i< fh->f_size; i++){
 	min_st_offset = OMPIO_MIN(min_st_offset, start_offsets[i]);
 	max_end_offset = OMPIO_MAX(max_end_offset, end_offsets[i]);
-	
+
     }
 
-    fd_size = ((max_end_offset - min_st_offset + 1) + nprocs_for_coll - 1)/nprocs_for_coll; 
-    
+    fd_size = ((max_end_offset - min_st_offset + 1) + nprocs_for_coll - 1)/nprocs_for_coll;
+
     if (fd_size < min_fd_size)
 	fd_size = min_fd_size;
-    
+
     *fd_st_ptr = (OMPI_MPI_OFFSET_TYPE *)
-	malloc(nprocs_for_coll*sizeof(OMPI_MPI_OFFSET_TYPE)); 
+	malloc(nprocs_for_coll*sizeof(OMPI_MPI_OFFSET_TYPE));
 
     if ( NULL == *fd_st_ptr ) {
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
     *fd_end_ptr = (OMPI_MPI_OFFSET_TYPE *)
-	malloc(nprocs_for_coll*sizeof(OMPI_MPI_OFFSET_TYPE)); 
+	malloc(nprocs_for_coll*sizeof(OMPI_MPI_OFFSET_TYPE));
 
     if ( NULL == *fd_end_ptr ) {
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    
+
     fd_start = *fd_st_ptr;
     fd_end = *fd_end_ptr;
-    
-    
+
+
     if (striping_unit > 0){
       /* Lock Boundary based domain partitioning */
 	int rem_front, rem_back;
 	OMPI_MPI_OFFSET_TYPE end_off;
-	
+
 	fd_start[0] = min_st_offset;
         end_off     = fd_start[0] + fd_size;
         rem_front   = end_off % striping_unit;
         rem_back    = striping_unit - rem_front;
-        if (rem_front < rem_back) 
+        if (rem_front < rem_back)
 		end_off -= rem_front;
-        else                      
+        else
 		end_off += rem_back;
         fd_end[0] = end_off - 1;
-    
+
 	/* align fd_end[i] to the nearest file lock boundary */
         for (i=1; i<nprocs_for_coll; i++) {
             fd_start[i] = fd_end[i-1] + 1;
             end_off     = min_st_offset + fd_size * (i+1);
             rem_front   = end_off % striping_unit;
             rem_back    = striping_unit - rem_front;
-            if (rem_front < rem_back) 
+            if (rem_front < rem_back)
 		    end_off -= rem_front;
-            else                      
+            else
 		    end_off += rem_back;
             fd_end[i] = end_off - 1;
         }
@@ -117,52 +120,52 @@ int mca_fcoll_two_phase_domain_partition (mca_io_ompio_file_t *fh,
     else{
 	fd_start[0] = min_st_offset;
         fd_end[0] = min_st_offset + fd_size - 1;
-	
+
         for (i=1; i<nprocs_for_coll; i++) {
             fd_start[i] = fd_end[i-1] + 1;
             fd_end[i] = fd_start[i] + fd_size - 1;
         }
 
     }
-    
+
     for (i=0; i<nprocs_for_coll; i++) {
 	if (fd_start[i] > max_end_offset)
 	    fd_start[i] = fd_end[i] = -1;
 	if (fd_end[i] > max_end_offset)
 	    fd_end[i] = max_end_offset;
     }
-    
+
     *fd_size_ptr = fd_size;
     *min_st_offset_ptr = min_st_offset;
- 
+
     return OMPI_SUCCESS;
 }
 
 
 
-int mca_fcoll_two_phase_calc_aggregator(mca_io_ompio_file_t *fh,
-					OMPI_MPI_OFFSET_TYPE off, 
+int mca_fcoll_two_phase_calc_aggregator(ompio_file_t *fh,
+					OMPI_MPI_OFFSET_TYPE off,
 					OMPI_MPI_OFFSET_TYPE min_off,
 					OMPI_MPI_OFFSET_TYPE *len,
 					OMPI_MPI_OFFSET_TYPE fd_size,
 					OMPI_MPI_OFFSET_TYPE *fd_start,
 					OMPI_MPI_OFFSET_TYPE *fd_end,
 					int striping_unit,
-					int num_aggregators, 
+					int num_aggregators,
 					int *aggregator_list)
 {
 
 
     int rank_index, rank;
     OMPI_MPI_OFFSET_TYPE avail_bytes;
-    
+
     rank_index = (int) ((off - min_off + fd_size)/ fd_size - 1);
-    
+
     if (striping_unit > 0){
 	rank_index = 0;
 	while (off > fd_end[rank_index]) rank_index++;
     }
-    
+
 
     if (rank_index >= num_aggregators || rank_index < 0) {
        fprintf(stderr,
@@ -170,7 +173,7 @@ int mca_fcoll_two_phase_calc_aggregator(mca_io_ompio_file_t *fh,
        fprintf(stderr,
 	       "rank_index(%d) >= num_aggregators(%d)fd_size=%lld off=%lld\n",
 	       rank_index,num_aggregators,fd_size,off);
-       MPI_Abort(MPI_COMM_WORLD, 1);
+       ompi_mpi_abort(&ompi_mpi_comm_world.comm, 1);
     }
 
 
@@ -180,59 +183,59 @@ int mca_fcoll_two_phase_calc_aggregator(mca_io_ompio_file_t *fh,
     }
 
     rank = aggregator_list[rank_index];
-    
+
     #if 0
     printf("rank : %d, rank_index : %d\n",rank, rank_index);
     #endif
-    
+
     return rank;
 }
 
-int mca_fcoll_two_phase_calc_others_requests(mca_io_ompio_file_t *fh,
+int mca_fcoll_two_phase_calc_others_requests(ompio_file_t *fh,
 					     int count_my_req_procs,
 					     int *count_my_req_per_proc,
-					     mca_io_ompio_access_array_t *my_req,
+					     mca_common_ompio_access_array_t *my_req,
 					     int *count_others_req_procs_ptr,
-					     mca_io_ompio_access_array_t **others_req_ptr)
+					     mca_common_ompio_access_array_t **others_req_ptr)
 {
-    
+
 
     int *count_others_req_per_proc=NULL, count_others_req_procs;
     int i,j, ret=OMPI_SUCCESS;
     MPI_Request *requests=NULL;
-    mca_io_ompio_access_array_t *others_req=NULL;
-    
+    mca_common_ompio_access_array_t *others_req=NULL;
+
     count_others_req_per_proc = (int *)malloc(fh->f_size*sizeof(int));
 
     if ( NULL == count_others_req_per_proc ) {
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    
+
     /* Change it to the ompio specific alltoall in coll module : VVN*/
-    ret =  fh->f_comm->c_coll.coll_alltoall (count_my_req_per_proc,
+    ret =  fh->f_comm->c_coll->coll_alltoall (count_my_req_per_proc,
 					     1,
 					     MPI_INT,
-					     count_others_req_per_proc, 
+					     count_others_req_per_proc,
 					     1,
 					     MPI_INT,
-					     fh->f_comm, 
-					     fh->f_comm->c_coll.coll_alltoall_module);
+					     fh->f_comm,
+					     fh->f_comm->c_coll->coll_alltoall_module);
     if ( OMPI_SUCCESS != ret ) {
 	return ret;
     }
-    
+
 #if 0
     for( i = 0; i< fh->f_size; i++){
 	printf("my: %d, others: %d\n",count_my_req_per_proc[i],
 	       count_others_req_per_proc[i]);
-	
+
     }
 #endif
 
-    *others_req_ptr = (mca_io_ompio_access_array_t  *) malloc 
-	(fh->f_size*sizeof(mca_io_ompio_access_array_t)); 
+    *others_req_ptr = (mca_common_ompio_access_array_t  *) malloc
+	(fh->f_size*sizeof(mca_common_ompio_access_array_t));
     others_req = *others_req_ptr;
-    
+
     count_others_req_procs = 0;
     for (i=0; i<fh->f_size; i++) {
 	if (count_others_req_per_proc[i]) {
@@ -242,23 +245,23 @@ int mca_fcoll_two_phase_calc_others_requests(mca_io_ompio_file_t *fh,
 	    others_req[i].lens = (int *)
 		malloc(count_others_req_per_proc[i]*sizeof(int));
 	    others_req[i].mem_ptrs = (MPI_Aint *)
-		malloc(count_others_req_per_proc[i]*sizeof(MPI_Aint)); 
+		malloc(count_others_req_per_proc[i]*sizeof(MPI_Aint));
 	    count_others_req_procs++;
 	}
 	else
 	    others_req[i].count = 0;
     }
-    
-    
+
+
     requests = (MPI_Request *)
 	malloc(1+2*(count_my_req_procs+count_others_req_procs)*
-	       sizeof(MPI_Request)); 
+	       sizeof(MPI_Request));
 
     if ( NULL == requests ) {
 	ret = OMPI_ERR_OUT_OF_RESOURCE;
 	goto exit;
     }
-    
+
     j = 0;
     for (i=0; i<fh->f_size; i++){
 	if (others_req[i].count){
@@ -272,7 +275,7 @@ int mca_fcoll_two_phase_calc_others_requests(mca_io_ompio_file_t *fh,
 	    if ( OMPI_SUCCESS != ret  ) {
 		goto exit;
 	    }
-	    	    
+
 	    j++;
 
 	    ret = MCA_PML_CALL(irecv(others_req[i].lens,
@@ -285,7 +288,7 @@ int mca_fcoll_two_phase_calc_others_requests(mca_io_ompio_file_t *fh,
 	    if ( OMPI_SUCCESS != ret  ) {
 		goto exit;
 	    }
-		       
+
 	    j++;
 	}
     }
@@ -298,9 +301,9 @@ int mca_fcoll_two_phase_calc_others_requests(mca_io_ompio_file_t *fh,
 				     OMPI_OFFSET_DATATYPE,
 				     i,
 				     i+fh->f_rank,
-				     MCA_PML_BASE_SEND_STANDARD, 
+				     MCA_PML_BASE_SEND_STANDARD,
 				     fh->f_comm,
-				     &requests[j]));	
+				     &requests[j]));
 	    if ( OMPI_SUCCESS != ret  ) {
 		goto exit;
 	    }
@@ -311,17 +314,17 @@ int mca_fcoll_two_phase_calc_others_requests(mca_io_ompio_file_t *fh,
 				     MPI_INT,
 				     i,
 				     i+fh->f_rank+1,
-				     MCA_PML_BASE_SEND_STANDARD, 
+				     MCA_PML_BASE_SEND_STANDARD,
 				     fh->f_comm,
-				     &requests[j]));	
+				     &requests[j]));
 	    if ( OMPI_SUCCESS != ret  ) {
 		goto exit;
 	    }
-    
+
 	    j++;
 	}
     }
-    
+
     if (j) {
 	ret = ompi_request_wait_all ( j, requests, MPI_STATUSES_IGNORE );
 	if ( OMPI_SUCCESS != ret  ) {
@@ -344,7 +347,7 @@ exit:
 }
 
 
-int mca_fcoll_two_phase_calc_my_requests (mca_io_ompio_file_t *fh,
+int mca_fcoll_two_phase_calc_my_requests (ompio_file_t *fh,
 					  struct iovec *offset_len,
 					  int contig_access_count,
 					  OMPI_MPI_OFFSET_TYPE min_st_offset,
@@ -353,55 +356,55 @@ int mca_fcoll_two_phase_calc_my_requests (mca_io_ompio_file_t *fh,
 					  OMPI_MPI_OFFSET_TYPE fd_size,
 					  int *count_my_req_procs_ptr,
 					  int **count_my_req_per_proc_ptr,
-					  mca_io_ompio_access_array_t **my_req_ptr,
+					  mca_common_ompio_access_array_t **my_req_ptr,
 					  size_t **buf_indices,
 					  int striping_unit,
 					  int num_aggregators,
 					  int *aggregator_list)
 {
-    
+    int ret = MPI_SUCCESS;
     int *count_my_req_per_proc, count_my_req_procs;
     size_t *buf_idx = NULL;
     int i, l, proc;
     OMPI_MPI_OFFSET_TYPE fd_len, rem_len, curr_idx, off;
-    mca_io_ompio_access_array_t *my_req;
-    
-    
-    *count_my_req_per_proc_ptr = (int*)malloc(fh->f_size*sizeof(int)); 
-    
-    if ( NULL == count_my_req_per_proc_ptr ){
+    mca_common_ompio_access_array_t *my_req = NULL;
+
+
+    *count_my_req_per_proc_ptr = (int*)malloc(fh->f_size*sizeof(int));
+
+    if ( NULL == *count_my_req_per_proc_ptr ){
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
     count_my_req_per_proc = *count_my_req_per_proc_ptr;
-    
+
     for (i=0;i<fh->f_size;i++){
 	count_my_req_per_proc[i] = 0;
     }
-        
+
     buf_idx = (size_t *) malloc (fh->f_size * sizeof(size_t));
-    
-    if ( NULL == buf_idx ){ 
+
+    if ( NULL == buf_idx ){
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    
+
     for (i=0; i < fh->f_size; i++) buf_idx[i] = -1;
-    
+
     for (i=0;i<contig_access_count; i++){
 
 	if (offset_len[i].iov_len==0)
 	    continue;
 	off = (OMPI_MPI_OFFSET_TYPE)(intptr_t)offset_len[i].iov_base;
 	fd_len = (OMPI_MPI_OFFSET_TYPE)offset_len[i].iov_len;
-	proc = mca_fcoll_two_phase_calc_aggregator(fh, off, min_st_offset, &fd_len, fd_size, 
+	proc = mca_fcoll_two_phase_calc_aggregator(fh, off, min_st_offset, &fd_len, fd_size,
 					     fd_start, fd_end, striping_unit, num_aggregators,aggregator_list);
 	count_my_req_per_proc[proc]++;
 	rem_len = offset_len[i].iov_len - fd_len;
-	
+
 	while (rem_len != 0) {
 	    off += fd_len; /* point to first remaining byte */
 	    fd_len = rem_len; /* save remaining size, pass to calc */
-	    proc = mca_fcoll_two_phase_calc_aggregator(fh, off, min_st_offset, &fd_len, 
+	    proc = mca_fcoll_two_phase_calc_aggregator(fh, off, min_st_offset, &fd_len,
 						 fd_size, fd_start, fd_end, striping_unit,
 						 num_aggregators, aggregator_list);
 
@@ -410,37 +413,40 @@ int mca_fcoll_two_phase_calc_my_requests (mca_io_ompio_file_t *fh,
 	}
 
     }
-    
+
 /*    printf("%d: fh->f_size : %d\n", fh->f_rank,fh->f_size);*/
-    *my_req_ptr =  (mca_io_ompio_access_array_t *)
-	malloc (fh->f_size * sizeof(mca_io_ompio_access_array_t));
+    *my_req_ptr =  (mca_common_ompio_access_array_t *)
+	malloc (fh->f_size * sizeof(mca_common_ompio_access_array_t));
     if ( NULL == *my_req_ptr ) {
-	return OMPI_ERR_OUT_OF_RESOURCE;
+        ret = OMPI_ERR_OUT_OF_RESOURCE;
+        goto err_exit;
     }
     my_req = *my_req_ptr;
-    
+
     count_my_req_procs = 0;
     for (i = 0; i < fh->f_size; i++){
 	if(count_my_req_per_proc[i]) {
 	    my_req[i].offsets = (OMPI_MPI_OFFSET_TYPE *)
 		malloc(count_my_req_per_proc[i] * sizeof(OMPI_MPI_OFFSET_TYPE));
-	    
+
 	    if ( NULL == my_req[i].offsets ) {
-		return OMPI_ERR_OUT_OF_RESOURCE;
+	        ret =  OMPI_ERR_OUT_OF_RESOURCE;
+                goto err_exit;
 	    }
 
 	    my_req[i].lens = (int *)
 		malloc(count_my_req_per_proc[i] * sizeof(int));
 
 	    if ( NULL == my_req[i].lens ) {
-		return OMPI_ERR_OUT_OF_RESOURCE;
+	        ret =  OMPI_ERR_OUT_OF_RESOURCE;
+                goto err_exit;
 	    }
 	    count_my_req_procs++;
 	}
-	my_req[i].count = 0; 
+	my_req[i].count = 0;
     }
     curr_idx = 0;
-    for (i=0; i<contig_access_count; i++) { 
+    for (i=0; i<contig_access_count; i++) {
 	if ((int)offset_len[i].iov_len == 0)
 	    continue;
 	off = (OMPI_MPI_OFFSET_TYPE)(intptr_t)offset_len[i].iov_base;
@@ -467,11 +473,11 @@ int mca_fcoll_two_phase_calc_my_requests (mca_io_ompio_file_t *fh,
 						       fd_end, striping_unit,
 						       num_aggregators,
 						       aggregator_list);
-	    
+
 	    if (buf_idx[proc] == (size_t) -1){
 		buf_idx[proc] = (int) curr_idx;
 	    }
-		    
+
 	    l = my_req[proc].count;
 	    curr_idx += fd_len;
 	    rem_len -= fd_len;
@@ -479,15 +485,15 @@ int mca_fcoll_two_phase_calc_my_requests (mca_io_ompio_file_t *fh,
 	    my_req[proc].offsets[l] = off;
 	    my_req[proc].lens[l] = (int) fd_len;
 	    my_req[proc].count++;
- 
+
 	}
-	
+
     }
-    
+
   #if 0
     for (i=0; i<fh->f_size; i++) {
 	if (count_my_req_per_proc[i] > 0) {
-	    fprintf(stdout, "data needed from %d (count = %d):\n", i, 
+	    fprintf(stdout, "data needed from %d (count = %d):\n", i,
 		    my_req[i].count);
 	    for (l=0; l < my_req[i].count; l++) {
 		fprintf(stdout, " %d: off[%d] = %lld, len[%d] = %d\n", fh->f_rank, l,
@@ -497,11 +503,26 @@ int mca_fcoll_two_phase_calc_my_requests (mca_io_ompio_file_t *fh,
 	}
     }
 #endif
-    
-    
+
+
     *count_my_req_procs_ptr = count_my_req_procs;
     *buf_indices = buf_idx;
-    
-    return OMPI_SUCCESS;
+
+    return ret;
+err_exit:
+    if (NULL != my_req) {
+        for (i = 0; i < fh->f_size; i++) {
+            if (NULL != my_req[i].offsets) {
+                free(my_req[i].offsets);
+            }
+            if (NULL != my_req[i].lens) {
+                free(my_req[i].lens);
+            }
+        }
+    }
+    if (NULL != buf_idx) {
+        free(buf_idx);
+    }
+    return ret;
 }
-/*Two-phase support functions ends here!*/				    
+/*Two-phase support functions ends here!*/
