@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2013 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -12,8 +12,9 @@
  *                         All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2011      NVIDIA Corporation.  All rights reserved.
- * Copyright (c) 2013      Research Organization for Information Science 
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2013-2018 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2017      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,10 +26,7 @@
 
 #include <stddef.h>
 #include <stdio.h>
-
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#endif
 
 #include "opal/prefetch.h"
 #include "opal/util/arch.h"
@@ -46,9 +44,6 @@
     CONVERTOR->cbmemcpy( (DST), (SRC), (BLENGTH), (CONVERTOR) )
 #endif
 
-extern int opal_convertor_create_stack_with_pos_general( opal_convertor_t* convertor,
-                                                         int starting_point, const int* sizes );
-
 static void opal_convertor_construct( opal_convertor_t* convertor )
 {
     convertor->pStack         = convertor->static_stack;
@@ -214,7 +209,7 @@ opal_convertor_t* opal_convertor_create( int32_t remote_arch, int32_t mode )
         assert( (CONVERTOR)->bConverted < (CONVERTOR)->local_size );    \
     } while(0)
 
-/** 
+/**
  * Return 0 if everything went OK and if there is still room before the complete
  *          conversion of the data (need additional call with others input buffers )
  *        1 if everything went fine and the data was completly converted
@@ -229,7 +224,7 @@ int32_t opal_convertor_pack( opal_convertor_t* pConv,
     if( OPAL_LIKELY(pConv->flags & CONVERTOR_NO_OP) ) {
         /**
          * We are doing conversion on a contiguous datatype on a homogeneous
-         * environment. The convertor contain minimal informations, we only
+         * environment. The convertor contain minimal information, we only
          * use the bConverted to manage the conversion.
          */
         uint32_t i;
@@ -335,8 +330,8 @@ static inline int opal_convertor_create_stack_with_pos_contig( opal_convertor_t*
     dt_stack_t* pStack;   /* pointer to the position on the stack */
     const opal_datatype_t* pData = pConvertor->pDesc;
     dt_elem_desc_t* pElems;
-    uint32_t count;
-    OPAL_PTRDIFF_TYPE extent;
+    size_t count;
+    ptrdiff_t extent;
 
     pStack = pConvertor->pStack;
     /**
@@ -345,7 +340,7 @@ static inline int opal_convertor_create_stack_with_pos_contig( opal_convertor_t*
      */
     pElems = pConvertor->use_desc->desc;
 
-    count = (uint32_t)(starting_point / pData->size);
+    count = starting_point / pData->size;
     extent = pData->ub - pData->lb;
 
     pStack[0].type     = OPAL_DATATYPE_LOOP;  /* the first one is always the loop */
@@ -354,7 +349,7 @@ static inline int opal_convertor_create_stack_with_pos_contig( opal_convertor_t*
     pStack[0].disp     = count * extent;
 
     /* now compute the number of pending bytes */
-    count = (uint32_t)(starting_point - count * pData->size);
+    count = starting_point - count * pData->size;
     /**
      * We save the current displacement starting from the begining
      * of this data.
@@ -362,13 +357,12 @@ static inline int opal_convertor_create_stack_with_pos_contig( opal_convertor_t*
     if( OPAL_LIKELY(0 == count) ) {
         pStack[1].type     = pElems->elem.common.type;
         pStack[1].count    = pElems->elem.count;
-        pStack[1].disp     = pElems->elem.disp;
     } else {
         pStack[1].type  = OPAL_DATATYPE_UINT1;
         pStack[1].count = pData->size - count;
-        pStack[1].disp  = pData->true_lb + count;
     }
-    pStack[1].index    = 0;  /* useless */
+    pStack[1].disp  = count;
+    pStack[1].index = 0;  /* useless */
 
     pConvertor->bConverted = starting_point;
     pConvertor->stack_pos = 1;
@@ -400,13 +394,16 @@ int opal_convertor_create_stack_at_begining( opal_convertor_t* convertor,
     pStack[0].index = -1;
     pStack[0].count = convertor->count;
     pStack[0].disp  = 0;
+    pStack[0].type  = OPAL_DATATYPE_LOOP;
 
     pStack[1].index = 0;
     pStack[1].disp = 0;
     if( pElems[0].elem.common.type == OPAL_DATATYPE_LOOP ) {
         pStack[1].count = pElems[0].loop.loops;
+        pStack[1].type  = OPAL_DATATYPE_LOOP;
     } else {
         pStack[1].count = pElems[0].elem.count;
+        pStack[1].type  = pElems[0].elem.common.type;
     }
     return OPAL_SUCCESS;
 }
@@ -418,17 +415,19 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
     int32_t rc;
 
     /**
-     * If we plan to rollback the convertor then first we have to set it
-     * at the beginning.
+     * create_stack_with_pos_contig always set the position relative to the ZERO
+     * position, so there is no need for special handling. In all other cases,
+     * if we plan to rollback the convertor then first we have to reset it at
+     * the beginning.
      */
-    if( (0 == (*position)) || ((*position) < convertor->bConverted) ) {
-        rc = opal_convertor_create_stack_at_begining( convertor, opal_datatype_local_sizes );
-        if( 0 == (*position) ) return rc;
-    }
     if( OPAL_LIKELY(convertor->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS) ) {
         rc = opal_convertor_create_stack_with_pos_contig( convertor, (*position),
                                                           opal_datatype_local_sizes );
     } else {
+        if( (0 == (*position)) || ((*position) < convertor->bConverted) ) {
+            rc = opal_convertor_create_stack_at_begining( convertor, opal_datatype_local_sizes );
+            if( 0 == (*position) ) return rc;
+        }
         rc = opal_convertor_generic_simple_position( convertor, position );
         /**
          * If we have a non-contigous send convertor don't allow it move in the middle
@@ -446,35 +445,56 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
     return rc;
 }
 
+static size_t
+opal_datatype_compute_remote_size( const opal_datatype_t* pData,
+                                   const size_t* sizes )
+{
+    uint32_t typeMask = pData->bdt_used;
+    size_t length = 0;
+
+    if (opal_datatype_is_predefined(pData)) {
+        return sizes[pData->desc.desc->elem.common.type];
+    }
+
+    if( OPAL_UNLIKELY(NULL == pData->ptypes) ) {
+        /* Allocate and fill the array of types used in the datatype description */
+        opal_datatype_compute_ptypes( (opal_datatype_t*)pData );
+    }
+
+    for( int i = OPAL_DATATYPE_FIRST_TYPE; typeMask && (i < OPAL_DATATYPE_MAX_PREDEFINED); i++ ) {
+        if( typeMask & ((uint32_t)1 << i) ) {
+            length += (pData->ptypes[i] * sizes[i]);
+            typeMask ^= ((uint32_t)1 << i);
+        }
+    }
+    return length;
+}
 
 /**
- * Compute the remote size.
+ * Compute the remote size. If necessary remove the homogeneous flag
+ * and redirect the convertor description toward the non-optimized
+ * datatype representation.
  */
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-#define OPAL_CONVERTOR_COMPUTE_REMOTE_SIZE(convertor, datatype, bdt_mask) \
-{                                                                         \
-    if( OPAL_UNLIKELY(0 != (bdt_mask)) ) {                                \
-        opal_convertor_master_t* master;                                  \
-        int i;                                                            \
-        uint32_t mask = datatype->bdt_used;                               \
-        convertor->flags ^= CONVERTOR_HOMOGENEOUS;                        \
-        master = convertor->master;                                       \
-        convertor->remote_size = 0;                                       \
-        for( i = OPAL_DATATYPE_FIRST_TYPE; mask && (i < OPAL_DATATYPE_MAX_PREDEFINED); i++ ) { \
-            if( mask & ((uint32_t)1 << i) ) {                             \
-                convertor->remote_size += (datatype->btypes[i] *          \
-                                           master->remote_sizes[i]);      \
-                mask ^= ((uint32_t)1 << i);                               \
-            }                                                             \
-        }                                                                 \
-        convertor->remote_size *= convertor->count;                       \
-        convertor->use_desc = &(datatype->desc);                          \
-    }                                                                     \
+size_t opal_convertor_compute_remote_size( opal_convertor_t* pConvertor )
+{
+    opal_datatype_t* datatype = (opal_datatype_t*)pConvertor->pDesc;
+    
+    pConvertor->remote_size = pConvertor->local_size;
+    if( OPAL_UNLIKELY(datatype->bdt_used & pConvertor->master->hetero_mask) ) {
+        pConvertor->flags &= (~CONVERTOR_HOMOGENEOUS);
+        if (!(pConvertor->flags & CONVERTOR_SEND && pConvertor->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS)) {
+            pConvertor->use_desc = &(datatype->desc);
+        }
+        if( 0 == (pConvertor->flags & CONVERTOR_HAS_REMOTE_SIZE) ) {
+            /* This is for a single datatype, we must update it with the count */
+            pConvertor->remote_size = opal_datatype_compute_remote_size(datatype,
+                                                                        pConvertor->master->remote_sizes);
+            pConvertor->remote_size *= pConvertor->count;
+        }
+    }
+    pConvertor->flags |= CONVERTOR_HAS_REMOTE_SIZE;
+    return pConvertor->remote_size;
 }
-#else
-#define OPAL_CONVERTOR_COMPUTE_REMOTE_SIZE(convertor, datatype, bdt_mask) \
-    assert(0 == (bdt_mask))
-#endif  /* OPAL_ENABLE_HETEROGENEOUS_SUPPORT */
 
 /**
  * This macro will initialize a convertor based on a previously created
@@ -485,56 +505,50 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
  */
 #define OPAL_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf )  \
     {                                                                   \
-        uint32_t bdt_mask;                                              \
-                                                                        \
+        convertor->local_size = count * datatype->size;                 \
+        convertor->pBaseBuf   = (unsigned char*)pUserBuf;               \
+        convertor->count      = count;                                  \
+        convertor->pDesc      = (opal_datatype_t*)datatype;             \
+        convertor->bConverted = 0;                                      \
+        convertor->use_desc   = &(datatype->opt_desc);                  \
         /* If the data is empty we just mark the convertor as           \
          * completed. With this flag set the pack and unpack functions  \
          * will not do anything.                                        \
          */                                                             \
         if( OPAL_UNLIKELY((0 == count) || (0 == datatype->size)) ) {    \
-            convertor->flags |= OPAL_DATATYPE_FLAG_NO_GAPS | CONVERTOR_COMPLETED;  \
+            convertor->flags |= (OPAL_DATATYPE_FLAG_NO_GAPS | CONVERTOR_COMPLETED | CONVERTOR_HAS_REMOTE_SIZE); \
             convertor->local_size = convertor->remote_size = 0;         \
             return OPAL_SUCCESS;                                        \
         }                                                               \
-        /* Compute the local in advance */                              \
-        convertor->local_size = count * datatype->size;                 \
-        convertor->pBaseBuf   = (unsigned char*)pUserBuf;               \
-        convertor->count      = count;                                  \
                                                                         \
         /* Grab the datatype part of the flags */                       \
         convertor->flags     &= CONVERTOR_TYPE_MASK;                    \
         convertor->flags     |= (CONVERTOR_DATATYPE_MASK & datatype->flags); \
         convertor->flags     |= (CONVERTOR_NO_OP | CONVERTOR_HOMOGENEOUS); \
-        convertor->pDesc      = (opal_datatype_t*)datatype;             \
-        convertor->bConverted = 0;                                      \
-        /* By default consider the optimized description */             \
-        convertor->use_desc = &(datatype->opt_desc);                    \
                                                                         \
         convertor->remote_size = convertor->local_size;                 \
         if( OPAL_LIKELY(convertor->remoteArch == opal_local_arch) ) {   \
-            if( (convertor->flags & (CONVERTOR_WITH_CHECKSUM | OPAL_DATATYPE_FLAG_NO_GAPS)) == OPAL_DATATYPE_FLAG_NO_GAPS ) { \
-                return OPAL_SUCCESS;                                    \
-            }                                                           \
-            if( ((convertor->flags & (CONVERTOR_WITH_CHECKSUM | OPAL_DATATYPE_FLAG_CONTIGUOUS)) \
-                 == OPAL_DATATYPE_FLAG_CONTIGUOUS) && (1 == count) ) {             \
+            if( !(convertor->flags & CONVERTOR_WITH_CHECKSUM) &&        \
+                ((convertor->flags & OPAL_DATATYPE_FLAG_NO_GAPS) || \
+                 ((convertor->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS) && (1 == count))) ) { \
                 return OPAL_SUCCESS;                                    \
             }                                                           \
         }                                                               \
                                                                         \
-        bdt_mask = datatype->bdt_used & convertor->master->hetero_mask; \
-        OPAL_CONVERTOR_COMPUTE_REMOTE_SIZE( convertor, datatype,        \
-                                            bdt_mask );                 \
+        assert( (convertor)->pDesc == (datatype) );                     \
+        opal_convertor_compute_remote_size( convertor );                \
         assert( NULL != convertor->use_desc->desc );                    \
         /* For predefined datatypes (contiguous) do nothing more */     \
         /* if checksum is enabled then always continue */               \
         if( ((convertor->flags & (CONVERTOR_WITH_CHECKSUM | OPAL_DATATYPE_FLAG_NO_GAPS)) \
-             == OPAL_DATATYPE_FLAG_NO_GAPS) &&                                     \
-            (convertor->flags & (CONVERTOR_SEND | CONVERTOR_HOMOGENEOUS)) ) { \
+             == OPAL_DATATYPE_FLAG_NO_GAPS) &&                          \
+            ((convertor->flags & (CONVERTOR_SEND | CONVERTOR_HOMOGENEOUS)) == \
+             (CONVERTOR_SEND | CONVERTOR_HOMOGENEOUS)) ) {              \
             return OPAL_SUCCESS;                                        \
         }                                                               \
         convertor->flags &= ~CONVERTOR_NO_OP;                           \
         {                                                               \
-            uint32_t required_stack_length = datatype->btypes[OPAL_DATATYPE_LOOP] + 1; \
+            uint32_t required_stack_length = datatype->loops + 1;       \
                                                                         \
             if( required_stack_length > convertor->stack_size ) {       \
                 assert(convertor->pStack == convertor->static_stack);   \
@@ -549,39 +563,40 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
 
 int32_t opal_convertor_prepare_for_recv( opal_convertor_t* convertor,
                                          const struct opal_datatype_t* datatype,
-                                         int32_t count,
+                                         size_t count,
                                          const void* pUserBuf )
 {
     /* Here I should check that the data is not overlapping */
 
     convertor->flags |= CONVERTOR_RECV;
 #if OPAL_CUDA_SUPPORT
-    mca_cuda_convertor_init(convertor, pUserBuf);
+    if (!( convertor->flags & CONVERTOR_SKIP_CUDA_INIT )) {
+        mca_cuda_convertor_init(convertor, pUserBuf);
+    }
 #endif
 
+    assert(! (convertor->flags & CONVERTOR_SEND));
     OPAL_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf );
 
     if( convertor->flags & CONVERTOR_WITH_CHECKSUM ) {
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         if( !(convertor->flags & CONVERTOR_HOMOGENEOUS) ) {
             convertor->fAdvance = opal_unpack_general_checksum;
-        } else
-#endif
-        if( convertor->pDesc->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
-            convertor->fAdvance = opal_unpack_homogeneous_contig_checksum;
         } else {
-            convertor->fAdvance = opal_generic_simple_unpack_checksum;
+            if( convertor->pDesc->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
+                convertor->fAdvance = opal_unpack_homogeneous_contig_checksum;
+            } else {
+                convertor->fAdvance = opal_generic_simple_unpack_checksum;
+            }
         }
     } else {
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         if( !(convertor->flags & CONVERTOR_HOMOGENEOUS) ) {
             convertor->fAdvance = opal_unpack_general;
-        } else
-#endif
-        if( convertor->pDesc->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
-            convertor->fAdvance = opal_unpack_homogeneous_contig;
         } else {
-            convertor->fAdvance = opal_generic_simple_unpack;
+            if( convertor->pDesc->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
+                convertor->fAdvance = opal_unpack_homogeneous_contig;
+            } else {
+                convertor->fAdvance = opal_generic_simple_unpack;
+            }
         }
     }
     return OPAL_SUCCESS;
@@ -590,35 +605,45 @@ int32_t opal_convertor_prepare_for_recv( opal_convertor_t* convertor,
 
 int32_t opal_convertor_prepare_for_send( opal_convertor_t* convertor,
                                          const struct opal_datatype_t* datatype,
-                                         int32_t count,
+                                         size_t count,
                                          const void* pUserBuf )
 {
     convertor->flags |= CONVERTOR_SEND;
 #if OPAL_CUDA_SUPPORT
-    mca_cuda_convertor_init(convertor, pUserBuf);
+    if (!( convertor->flags & CONVERTOR_SKIP_CUDA_INIT )) {
+        mca_cuda_convertor_init(convertor, pUserBuf);
+    }
 #endif
 
     OPAL_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf );
 
     if( convertor->flags & CONVERTOR_WITH_CHECKSUM ) {
-        if( datatype->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
-            if( ((datatype->ub - datatype->lb) == (OPAL_PTRDIFF_TYPE)datatype->size)
-                || (1 >= convertor->count) )
-                convertor->fAdvance = opal_pack_homogeneous_contig_checksum;
-            else
-                convertor->fAdvance = opal_pack_homogeneous_contig_with_gaps_checksum;
+        if( CONVERTOR_SEND_CONVERSION == (convertor->flags & (CONVERTOR_SEND_CONVERSION|CONVERTOR_HOMOGENEOUS)) ) {
+            convertor->fAdvance = opal_pack_general_checksum;
         } else {
-            convertor->fAdvance = opal_generic_simple_pack_checksum;
+            if( datatype->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
+                if( ((datatype->ub - datatype->lb) == (ptrdiff_t)datatype->size)
+                    || (1 >= convertor->count) )
+                    convertor->fAdvance = opal_pack_homogeneous_contig_checksum;
+                else
+                    convertor->fAdvance = opal_pack_homogeneous_contig_with_gaps_checksum;
+            } else {
+                convertor->fAdvance = opal_generic_simple_pack_checksum;
+            }
         }
     } else {
-        if( datatype->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
-            if( ((datatype->ub - datatype->lb) == (OPAL_PTRDIFF_TYPE)datatype->size)
-                || (1 >= convertor->count) )
-                convertor->fAdvance = opal_pack_homogeneous_contig;
-            else
-                convertor->fAdvance = opal_pack_homogeneous_contig_with_gaps;
+        if( CONVERTOR_SEND_CONVERSION == (convertor->flags & (CONVERTOR_SEND_CONVERSION|CONVERTOR_HOMOGENEOUS)) ) {
+            convertor->fAdvance = opal_pack_general;
         } else {
-            convertor->fAdvance = opal_generic_simple_pack;
+            if( datatype->flags & OPAL_DATATYPE_FLAG_CONTIGUOUS ) {
+                if( ((datatype->ub - datatype->lb) == (ptrdiff_t)datatype->size)
+                    || (1 >= convertor->count) )
+                    convertor->fAdvance = opal_pack_homogeneous_contig;
+                else
+                    convertor->fAdvance = opal_pack_homogeneous_contig_with_gaps;
+            } else {
+                convertor->fAdvance = opal_generic_simple_pack;
+            }
         }
     }
     return OPAL_SUCCESS;
@@ -674,15 +699,33 @@ int opal_convertor_clone( const opal_convertor_t* source,
 
 void opal_convertor_dump( opal_convertor_t* convertor )
 {
-    printf( "Convertor %p count %d stack position %d bConverted %ld\n", (void*)convertor,
-            convertor->count, convertor->stack_pos, (unsigned long)convertor->bConverted );
-    printf( "\tlocal_size %ld remote_size %ld flags %X stack_size %d pending_length %d\n",
-            (unsigned long)convertor->local_size, (unsigned long)convertor->remote_size,
-            convertor->flags, convertor->stack_size, convertor->partial_length );
+    opal_output( 0, "Convertor %p count %" PRIsize_t" stack position %d bConverted %" PRIsize_t "\n"
+                 "\tlocal_size %ld remote_size %ld flags %X stack_size %d pending_length %" PRIsize_t "\n"
+                 "\tremote_arch %u local_arch %u\n",
+                 (void*)convertor,
+                 convertor->count, convertor->stack_pos, convertor->bConverted,
+                 (unsigned long)convertor->local_size, (unsigned long)convertor->remote_size,
+                 convertor->flags, convertor->stack_size, convertor->partial_length,
+                 convertor->remoteArch, opal_local_arch );
+    if( convertor->flags & CONVERTOR_RECV ) opal_output( 0, "unpack ");
+    if( convertor->flags & CONVERTOR_SEND ) opal_output( 0, "pack ");
+    if( convertor->flags & CONVERTOR_SEND_CONVERSION ) opal_output( 0, "conversion ");
+    if( convertor->flags & CONVERTOR_HOMOGENEOUS ) opal_output( 0, "homogeneous " );
+    else opal_output( 0, "heterogeneous ");
+    if( convertor->flags & CONVERTOR_NO_OP ) opal_output( 0, "no_op ");
+    if( convertor->flags & CONVERTOR_WITH_CHECKSUM ) opal_output( 0, "checksum ");
+    if( convertor->flags & CONVERTOR_CUDA ) opal_output( 0, "CUDA ");
+    if( convertor->flags & CONVERTOR_CUDA_ASYNC ) opal_output( 0, "CUDA Async ");
+    if( convertor->flags & CONVERTOR_COMPLETED ) opal_output( 0, "COMPLETED ");
+
     opal_datatype_dump( convertor->pDesc );
-    printf( "Actual stack representation\n" );
-    opal_datatype_dump_stack( convertor->pStack, convertor->stack_pos,
-                              convertor->pDesc->desc.desc, convertor->pDesc->name );
+    if( !((0 == convertor->stack_pos) &&
+          ((size_t)convertor->pStack[convertor->stack_pos].index > convertor->pDesc->desc.length)) ) {
+        /* only if the convertor is completely initialized */
+        opal_output( 0, "Actual stack representation\n" );
+        opal_datatype_dump_stack( convertor->pStack, convertor->stack_pos,
+                                  convertor->pDesc->desc.desc, convertor->pDesc->name );
+    }
 }
 
 
@@ -691,11 +734,11 @@ void opal_datatype_dump_stack( const dt_stack_t* pStack, int stack_pos,
 {
     opal_output( 0, "\nStack %p stack_pos %d name %s\n", (void*)pStack, stack_pos, name );
     for( ; stack_pos >= 0; stack_pos-- ) {
-        opal_output( 0, "%d: pos %d count %d disp %ld ", stack_pos, pStack[stack_pos].index,
-                     (int)pStack[stack_pos].count, (long)pStack[stack_pos].disp );
+        opal_output( 0, "%d: pos %d count %" PRIsize_t " disp %ld ", stack_pos, pStack[stack_pos].index,
+                     pStack[stack_pos].count, pStack[stack_pos].disp );
         if( pStack->index != -1 )
-            opal_output( 0, "\t[desc count %d disp %ld extent %ld]\n",
-                         pDesc[pStack[stack_pos].index].elem.count,
+            opal_output( 0, "\t[desc count %lu disp %ld extent %ld]\n",
+                         (unsigned long)pDesc[pStack[stack_pos].index].elem.count,
                          (long)pDesc[pStack[stack_pos].index].elem.disp,
                          (long)pDesc[pStack[stack_pos].index].elem.extent );
         else
