@@ -12,10 +12,12 @@
  * Copyright (c) 2010-2012 Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
  * Copyright (c) 2014-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2018      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -45,7 +47,6 @@
 #include "opal/util/output.h"
 #include "opal/util/proc.h"
 #include "opal/runtime/opal.h"
-#include "opal/runtime/opal_cr.h"
 
 #include "orte/mca/rml/base/base.h"
 #include "orte/mca/routed/base/base.h"
@@ -58,17 +59,13 @@
 #include "orte/mca/odls/odls_types.h"
 #include "orte/mca/filem/base/base.h"
 #include "orte/mca/errmgr/base/base.h"
-#if OPAL_ENABLE_FT_CR == 1
-#include "orte/mca/snapc/base/base.h"
-#include "orte/mca/sstore/base/base.h"
-#endif
 #include "orte/mca/state/base/base.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
 #include "orte/util/name_fns.h"
 #include "orte/util/show_help.h"
+#include "opal/util/timings.h"
 
-#include "orte/runtime/orte_cr.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_wait.h"
 
@@ -80,6 +77,7 @@ int orte_ess_base_app_setup(bool db_restrict_local)
     char *error = NULL;
     opal_list_t transports;
 
+    OPAL_TIMING_ENV_INIT(ess_base_setup);
     /*
      * stdout/stderr buffering
      * If the user requested to override the default setting then do
@@ -122,6 +120,7 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_state_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "state_framework_open");
 
     /* open the errmgr */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_errmgr_base_framework, 0))) {
@@ -129,6 +128,7 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_errmgr_base_open";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "errmgr_framework_open");
 
     /* setup my session directory */
     if (orte_create_session_dirs) {
@@ -147,7 +147,25 @@ int orte_ess_base_app_setup(bool db_restrict_local)
            proc-specific session directory. */
         opal_output_set_output_file_info(orte_process_info.proc_session_dir,
                                          "output-", NULL, NULL);
+        /* register the directory for cleanup */
+        if (NULL != opal_pmix.register_cleanup) {
+            if (orte_standalone_operation) {
+                if (OPAL_SUCCESS != (ret = opal_pmix.register_cleanup(orte_process_info.top_session_dir, true, false, true))) {
+                    ORTE_ERROR_LOG(ret);
+                    error = "register cleanup";
+                    goto error;
+                }
+            } else {
+                if (OPAL_SUCCESS != (ret = opal_pmix.register_cleanup(orte_process_info.job_session_dir, true, false, false))) {
+                    ORTE_ERROR_LOG(ret);
+                    error = "register cleanup";
+                    goto error;
+                }
+            }
+        }
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "create_session_dirs");
+
     /* Setup the communication infrastructure */
     /* Routed system */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_routed_base_framework, 0))) {
@@ -160,6 +178,8 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_routed_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "routed_framework_open");
+
     /*
      * OOB Layer
      */
@@ -173,6 +193,8 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_oob_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "oob_framework_open");
+    
     /* Runtime Messaging Layer */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_rml_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -184,6 +206,8 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_rml_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "rml_framework_open");
+    
     /* if we have info on the HNP and local daemon, process it */
     if (NULL != orte_process_info.my_hnp_uri) {
         /* we have to set the HNP's name, even though we won't route messages directly
@@ -198,6 +222,8 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         }
     }
     if (NULL != orte_process_info.my_daemon_uri) {
+        opal_value_t val;
+
         /* extract the daemon's name so we can update the routing table */
         if (ORTE_SUCCESS != (ret = orte_rml_base_parse_uris(orte_process_info.my_daemon_uri,
                                                             ORTE_PROC_MY_DAEMON, NULL))) {
@@ -205,11 +231,25 @@ int orte_ess_base_app_setup(bool db_restrict_local)
             error = "orte_rml_parse_daemon";
             goto error;
         }
-        /* Set the contact info in the RML - this won't actually establish
-         * the connection, but just tells the RML how to reach the daemon
+        /* Set the contact info in the database - this won't actually establish
+         * the connection, but just tells us how to reach the daemon
          * if/when we attempt to send to it
          */
-        orte_rml.set_contact_info(orte_process_info.my_daemon_uri);
+        OBJ_CONSTRUCT(&val, opal_value_t);
+        val.key = OPAL_PMIX_PROC_URI;
+        val.type = OPAL_STRING;
+        val.data.string = orte_process_info.my_daemon_uri;
+        if (OPAL_SUCCESS != (ret = opal_pmix.store_local(ORTE_PROC_MY_DAEMON, &val))) {
+            ORTE_ERROR_LOG(ret);
+            val.key = NULL;
+            val.data.string = NULL;
+            OBJ_DESTRUCT(&val);
+            error = "store DAEMON URI";
+            goto error;
+        }
+        val.key = NULL;
+        val.data.string = NULL;
+        OBJ_DESTRUCT(&val);
     }
 
     /* setup the errmgr */
@@ -218,6 +258,7 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_errmgr_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "errmgr_select");
 
     /* get a conduit for our use - we never route IO over fabric */
     OBJ_CONSTRUCT(&transports, opal_list_t);
@@ -239,6 +280,7 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         goto error;
     }
     OPAL_LIST_DESTRUCT(&transports);
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "rml_open_conduit");
 
     /*
      * Group communications
@@ -253,45 +295,8 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_grpcomm_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "grpcomm_framework_open");
 
-#if OPAL_ENABLE_FT_CR == 1
-    /*
-     * Setup the SnapC
-     */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_snapc_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_sstore_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_snapc_base_select(ORTE_PROC_IS_HNP, ORTE_PROC_IS_APP))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_select";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_sstore_base_select())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_select";
-        goto error;
-    }
-    /* apps need the OPAL CR stuff */
-    opal_cr_set_enabled(true);
-#else
-    opal_cr_set_enabled(false);
-#endif
-    /* Initalize the CR setup
-     * Note: Always do this, even in non-FT builds.
-     * If we don't some user level tools may hang.
-     */
-    if (ORTE_SUCCESS != (ret = orte_cr_init())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_cr_init";
-        goto error;
-    }
     /* open the distributed file system */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_dfs_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -303,6 +308,8 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_dfs_base_select";
         goto error;
     }
+    OPAL_TIMING_ENV_NEXT(ess_base_setup, "dfs_framework_open");
+
     return ORTE_SUCCESS;
  error:
     orte_show_help("help-orte-runtime.txt",
@@ -313,13 +320,6 @@ int orte_ess_base_app_setup(bool db_restrict_local)
 
 int orte_ess_base_app_finalize(void)
 {
-    orte_cr_finalize();
-
-#if OPAL_ENABLE_FT_CR == 1
-    (void) mca_base_framework_close(&orte_snapc_base_framework);
-    (void) mca_base_framework_close(&orte_sstore_base_framework);
-#endif
-
     /* release the conduits */
     orte_rml.close_conduit(orte_mgmt_conduit);
     orte_rml.close_conduit(orte_coll_conduit);
@@ -341,7 +341,9 @@ int orte_ess_base_app_finalize(void)
     (void) mca_base_framework_close(&orte_oob_base_framework);
     (void) mca_base_framework_close(&orte_state_base_framework);
 
-    orte_session_dir_finalize(ORTE_PROC_MY_NAME);
+    if (NULL == opal_pmix.register_cleanup) {
+        orte_session_dir_finalize(ORTE_PROC_MY_NAME);
+    }
     /* cleanup the process info */
     orte_proc_info_finalize();
 
@@ -381,8 +383,7 @@ void orte_ess_base_app_abort(int status, bool report)
      * clean environment. Taken from orte_finalize():
      * - Assume errmgr cleans up child processes before we exit.
      */
-    /* CRS cleanup since it may have a named pipe and thread active */
-    orte_cr_finalize();
+
     /* If we were asked to report this termination, do so.
      * Since singletons don't start an HNP unless necessary, and
      * direct-launched procs don't have daemons at all, only send
