@@ -14,7 +14,8 @@
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -37,11 +38,11 @@
 #include "opal/dss/dss.h"
 #include "opal/mca/event/event.h"
 #include "opal/runtime/opal.h"
-#include "opal/runtime/opal_cr.h"
 #include "opal/mca/hwloc/base/base.h"
 #include "opal/mca/pmix/base/base.h"
 #include "opal/mca/pstat/base/base.h"
 #include "opal/util/arch.h"
+#include "opal/util/opal_environ.h"
 #include "opal/util/os_path.h"
 #include "opal/util/proc.h"
 
@@ -57,23 +58,17 @@
 #include "orte/mca/iof/base/base.h"
 #include "orte/mca/plm/base/base.h"
 #include "orte/mca/odls/base/base.h"
+#include "orte/mca/regx/base/base.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rmaps/base/base.h"
-#if OPAL_ENABLE_FT_CR == 1
-#include "orte/mca/snapc/base/base.h"
-#include "orte/mca/sstore/base/base.h"
-#endif
 #include "orte/mca/filem/base/base.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
 #include "orte/util/name_fns.h"
-#include "orte/util/nidmap.h"
-#include "orte/util/regex.h"
 #include "orte/util/show_help.h"
 #include "orte/mca/errmgr/base/base.h"
 #include "orte/mca/state/base/base.h"
 #include "orte/mca/state/state.h"
-#include "orte/runtime/orte_cr.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_quit.h"
@@ -87,9 +82,7 @@ static bool signals_set=false;
 static opal_event_t term_handler;
 static opal_event_t int_handler;
 static opal_event_t epipe_handler;
-static opal_event_t sigusr1_handler;
-static opal_event_t sigusr2_handler;
-char *log_path = NULL;
+static char *log_path = NULL;
 static void shutdown_signal(int fd, short flags, void *arg);
 static void epipe_signal_callback(int fd, short flags, void *arg);
 static void signal_forward_callback(int fd, short event, void *arg);
@@ -373,7 +366,9 @@ int orte_ess_base_orted_setup(void)
     }
     /* set the event base */
     opal_pmix_base_set_evbase(orte_event_base);
-    /* setup the PMIx server */
+    /* setup the PMIx server - we need this here in case the
+     * communications infrastructure wants to register
+     * information */
     if (ORTE_SUCCESS != (ret = pmix_server_init())) {
         /* the server code already barked, so let's be quiet */
         ret = ORTE_ERR_SILENT;
@@ -414,7 +409,12 @@ int orte_ess_base_orted_setup(void)
         goto error;
     }
 
+    /* it is now safe to start the pmix server */
+    pmix_server_start();
+
     if (NULL != orte_process_info.my_hnp_uri) {
+        opal_value_t val;
+
         /* extract the HNP's name so we can update the routing table */
         if (ORTE_SUCCESS != (ret = orte_rml_base_parse_uris(orte_process_info.my_hnp_uri,
                                                             ORTE_PROC_MY_HNP, NULL))) {
@@ -426,7 +426,21 @@ int orte_ess_base_orted_setup(void)
          * the connection, but just tells the RML how to reach the HNP
          * if/when we attempt to send to it
          */
-        orte_rml.set_contact_info(orte_process_info.my_hnp_uri);
+        OBJ_CONSTRUCT(&val, opal_value_t);
+        val.key = OPAL_PMIX_PROC_URI;
+        val.type = OPAL_STRING;
+        val.data.string = orte_process_info.my_hnp_uri;
+        if (OPAL_SUCCESS != (ret = opal_pmix.store_local(ORTE_PROC_MY_HNP, &val))) {
+            ORTE_ERROR_LOG(ret);
+            val.key = NULL;
+            val.data.string = NULL;
+            OBJ_DESTRUCT(&val);
+            error = "store HNP URI";
+            goto error;
+        }
+        val.key = NULL;
+        val.data.string = NULL;
+        OBJ_DESTRUCT(&val);
     }
 
     /* select the errmgr */
@@ -457,10 +471,7 @@ int orte_ess_base_orted_setup(void)
     }
     OPAL_LIST_DESTRUCT(&transports);
 
-     /* add our contact info to our proc object */
-     proc->rml_uri = orte_rml.get_contact_info();
-
-   /*
+    /*
      * Group communications
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_grpcomm_base_framework, 0))) {
@@ -502,9 +513,20 @@ int orte_ess_base_orted_setup(void)
     }
     if (ORTE_SUCCESS != (ret = orte_rmaps_base_select())) {
         ORTE_ERROR_LOG(ret);
-        error = "orte_rmaps_base_find_available";
+        error = "orte_rmaps_base_select";
         goto error;
     }
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_regx_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_regx_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = orte_regx_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_regx_base_select";
+        goto error;
+    }
+
 
     /* if a topology file was given, then the rmaps framework open
      * will have reset our topology. Ensure we always get the right
@@ -528,14 +550,17 @@ int orte_ess_base_orted_setup(void)
      * own port, which we need in order to construct the nidmap
      */
     if (NULL != orte_node_regex) {
-        if (ORTE_SUCCESS != (ret = orte_util_nidmap_parse(orte_node_regex))) {
+        if (ORTE_SUCCESS != (ret = orte_regx.nidmap_parse(orte_node_regex))) {
             ORTE_ERROR_LOG(ret);
             error = "construct nidmap";
             goto error;
         }
+        /* be sure to update the routing tree so any tree spawn operation
+         * properly gets the number of children underneath us */
+        orte_routed.update_routing_plan(NULL);
     }
 
-    if (orte_static_ports) {
+    if (orte_static_ports || orte_fwd_mpirun_port) {
         if (NULL == orte_node_regex) {
             /* we didn't get the node info */
             error = "cannot construct daemon map for static ports - no node map info";
@@ -545,7 +570,7 @@ int orte_ess_base_orted_setup(void)
          * build a nidmap from it - this will update the
          * routing plan as well
          */
-        if (ORTE_SUCCESS != (ret = orte_util_build_daemon_nidmap())) {
+        if (ORTE_SUCCESS != (ret = orte_regx.build_daemon_nidmap())) {
             ORTE_ERROR_LOG(ret);
             error = "construct daemon map from static ports";
             goto error;
@@ -596,46 +621,6 @@ int orte_ess_base_orted_setup(void)
         goto error;
     }
 
-#if OPAL_ENABLE_FT_CR == 1
-    /*
-     * Setup the SnapC
-     */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_snapc_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_sstore_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_snapc_base_select(!ORTE_PROC_IS_HNP, ORTE_PROC_IS_DAEMON))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_select";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_sstore_base_select())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_select";
-        goto error;
-    }
-
-    /* For daemons, ORTE doesn't need the OPAL CR stuff */
-    opal_cr_set_enabled(false);
-#else
-    opal_cr_set_enabled(false);
-#endif
-    /*
-     * Initalize the CR setup
-     * Note: Always do this, even in non-FT builds.
-     * If we don't some user level tools may hang.
-     */
-    if (ORTE_SUCCESS != (ret = orte_cr_init())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_cr_init";
-        goto error;
-    }
     /* setup the DFS framework */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_dfs_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -650,7 +635,7 @@ int orte_ess_base_orted_setup(void)
 
     return ORTE_SUCCESS;
 
- error:
+  error:
     orte_show_help("help-orte-runtime.txt",
                    "orte_init:startup:internal-failure",
                    true, error, ORTE_ERROR_NAME(ret), ret);
