@@ -9,11 +9,13 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2013 Los Alamos National Security, LLC. 
+ * Copyright (c) 2006-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -23,7 +25,7 @@
  * In windows, many of the socket functions return an EWOULDBLOCK
  * instead of things like EAGAIN, EINPROGRESS, etc. It has been
  * verified that this will not conflict with other error codes that
- * are returned by these functions under UNIX/Linux environments 
+ * are returned by these functions under UNIX/Linux environments
  */
 
 #include "orte_config.h"
@@ -55,6 +57,7 @@
 #include "opal/util/if.h"
 #include "opal/util/net.h"
 #include "opal/util/argv.h"
+#include "opal/util/fd.h"
 #include "opal/class/opal_hash_table.h"
 #include "opal/class/opal_list.h"
 
@@ -63,6 +66,7 @@
 #include "orte/util/name_fns.h"
 #include "orte/util/parse_options.h"
 #include "orte/util/show_help.h"
+#include "orte/util/threads.h"
 #include "orte/runtime/orte_globals.h"
 
 #include "orte/mca/oob/tcp/oob_tcp.h"
@@ -121,11 +125,25 @@ int orte_oob_tcp_start_listening(void)
     }
 #endif
 
-    /* if I am the HNP, create a separate event base for the
-     * listening thread so we can harvest connection requests
-     * as rapidly as possible
+    /* if I am the HNP, start a listening thread so we can
+     * harvest connection requests as rapidly as possible
      */
     if (ORTE_PROC_IS_HNP) {
+        if (0 > pipe(mca_oob_tcp_component.stop_thread)) {
+            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+            return ORTE_ERR_OUT_OF_RESOURCE;
+        }
+
+        /* Make sure the pipe FDs are set to close-on-exec so that
+           they don't leak into children */
+        if (opal_fd_set_cloexec(mca_oob_tcp_component.stop_thread[0]) != OPAL_SUCCESS ||
+            opal_fd_set_cloexec(mca_oob_tcp_component.stop_thread[1]) != OPAL_SUCCESS) {
+            close(mca_oob_tcp_component.stop_thread[0]);
+            close(mca_oob_tcp_component.stop_thread[1]);
+            ORTE_ERROR_LOG(ORTE_ERR_IN_ERRNO);
+            return ORTE_ERR_IN_ERRNO;
+        }
+
         mca_oob_tcp_component.listen_thread_active = true;
         mca_oob_tcp_component.listen_thread.t_run = listen_thread;
         mca_oob_tcp_component.listen_thread.t_arg = NULL;
@@ -139,12 +157,13 @@ int orte_oob_tcp_start_listening(void)
     /* otherwise, setup to listen via the event lib */
     OPAL_LIST_FOREACH(listener, &mca_oob_tcp_component.listeners, mca_oob_tcp_listener_t) {
         listener->ev_active = true;
-        opal_event_set(orte_event_base, &listener->event,
+        opal_event_set(orte_oob_base.ev_base, &listener->event,
                        listener->sd,
                        OPAL_EV_READ|OPAL_EV_PERSIST,
                        connection_event_handler,
                        0);
         opal_event_set_priority(&listener->event, ORTE_MSG_PRI);
+        ORTE_POST_OBJECT(listener);
         opal_event_add(&listener->event, 0);
     }
 
@@ -152,8 +171,8 @@ int orte_oob_tcp_start_listening(void)
 }
 
 /*
- * Create an IPv4 listen socket and bind to all interfaces.  
- * 
+ * Create an IPv4 listen socket and bind to all interfaces.
+ *
  * At one time, this also registered a callback with the event library
  * for when connections were received on the listen socket.  This is
  * no longer the case -- the caller must register any events required.
@@ -256,8 +275,9 @@ static int create_listen(void)
     if (NULL == ports) {
         return ORTE_ERROR;
     }
-    
+
     /* get the address info for this interface */
+    memset(&inaddr, 0, sizeof(inaddr));
     ((struct sockaddr_in*) &inaddr)->sin_family = AF_INET;
     ((struct sockaddr_in*) &inaddr)->sin_addr.s_addr = INADDR_ANY;
     addrlen = sizeof(struct sockaddr_in);
@@ -278,20 +298,18 @@ static int create_listen(void)
         port = htons(port);
 
         ((struct sockaddr_in*) &inaddr)->sin_port = port;
-        
+
         /* create a listen socket for incoming connections on this port */
         sd = socket(AF_INET, SOCK_STREAM, 0);
         if (sd < 0) {
             if (EAFNOSUPPORT != opal_socket_errno) {
-                opal_output(0,"mca_oob_tcp_component_init: socket() failed: %s (%d)", 
+                opal_output(0,"mca_oob_tcp_component_init: socket() failed: %s (%d)",
                             strerror(opal_socket_errno), opal_socket_errno);
             }
+            opal_argv_free(ports);
             return ORTE_ERR_IN_ERRNO;
         }
 
-        /* setup socket options */
-        orte_oob_tcp_set_socket_options(sd);
-
         /* Enable/disable reusing ports */
         if (orte_static_ports) {
             flags = 1;
@@ -306,7 +324,18 @@ static int create_listen(void)
             opal_argv_free(ports);
             return ORTE_ERROR;
         }
-    
+
+        /* Set the socket to close-on-exec so that no children inherit
+           this FD */
+        if (opal_fd_set_cloexec(sd) != OPAL_SUCCESS) {
+            opal_output(0, "mca_oob_tcp_create_listen: unable to set the "
+                        "listening socket to CLOEXEC (%s:%d)\n",
+                        strerror(opal_socket_errno), opal_socket_errno);
+            CLOSE_THE_SOCKET(sd);
+            opal_argv_free(ports);
+            return ORTE_ERROR;
+        }
+
         if (bind(sd, (struct sockaddr*)&inaddr, addrlen) < 0) {
             if( (EADDRINUSE == opal_socket_errno) || (EADDRNOTAVAIL == opal_socket_errno) ) {
                 continue;
@@ -322,35 +351,47 @@ static int create_listen(void)
         }
         /* resolve assigned port */
         if (getsockname(sd, (struct sockaddr*)&inaddr, &addrlen) < 0) {
-            opal_output(0, "mca_oob_tcp_create_listen: getsockname(): %s (%d)", 
+            opal_output(0, "mca_oob_tcp_create_listen: getsockname(): %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
             CLOSE_THE_SOCKET(sd);
+            opal_argv_free(ports);
             return ORTE_ERROR;
         }
-        
+
         /* setup listen backlog to maximum allowed by kernel */
         if (listen(sd, SOMAXCONN) < 0) {
-            opal_output(0, "mca_oob_tcp_component_init: listen(): %s (%d)", 
+            opal_output(0, "mca_oob_tcp_component_init: listen(): %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
+            CLOSE_THE_SOCKET(sd);
+            opal_argv_free(ports);
             return ORTE_ERROR;
         }
-        
+
         /* set socket up to be non-blocking, otherwise accept could block */
         if ((flags = fcntl(sd, F_GETFL, 0)) < 0) {
-            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_GETFL) failed: %s (%d)", 
+            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_GETFL) failed: %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
+            CLOSE_THE_SOCKET(sd);
+            opal_argv_free(ports);
             return ORTE_ERROR;
         }
         flags |= O_NONBLOCK;
         if (fcntl(sd, F_SETFL, flags) < 0) {
-            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_SETFL) failed: %s (%d)", 
+            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_SETFL) failed: %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
+            CLOSE_THE_SOCKET(sd);
+            opal_argv_free(ports);
             return ORTE_ERROR;
         }
 
         /* add this port to our connections */
         conn = OBJ_NEW(mca_oob_tcp_listener_t);
         conn->sd = sd;
+        conn->port = ntohs(((struct sockaddr_in*) &inaddr)->sin_port);
+        if (0 == orte_process_info.my_port) {
+            /* save the first one */
+            orte_process_info.my_port = conn->port;
+        }
         opal_list_append(&mca_oob_tcp_component.listeners, &conn->item);
         /* and to our ports */
         asprintf(&tconn, "%d", ntohs(((struct sockaddr_in*) &inaddr)->sin_port));
@@ -367,25 +408,24 @@ static int create_listen(void)
             break;
         }
     }
+    /* done with this, so release it */
+    opal_argv_free(ports);
+
     if (0 == opal_list_get_size(&mca_oob_tcp_component.listeners)) {
         /* cleanup */
         if (0 <= sd) {
             CLOSE_THE_SOCKET(sd);
         }
-        opal_argv_free(ports);
         return ORTE_ERR_SOCKET_NOT_AVAILABLE;
     }
 
-    /* done with this, so release it */
-    opal_argv_free(ports);
-    
     return ORTE_SUCCESS;
 }
 
 #if OPAL_ENABLE_IPV6
 /*
- * Create an IPv6 listen socket and bind to all interfaces.  
- * 
+ * Create an IPv6 listen socket and bind to all interfaces.
+ *
  * At one time, this also registered a callback with the event library
  * for when connections were received on the listen socket.  This is
  * no longer the case -- the caller must register any events required.
@@ -488,8 +528,9 @@ static int create_listen6(void)
     if (NULL == ports) {
         return ORTE_ERROR;
     }
-    
+
     /* get the address info for this interface */
+    memset(&inaddr, 0, sizeof(inaddr));
     ((struct sockaddr_in6*) &inaddr)->sin6_family = AF_INET6;
     ((struct sockaddr_in6*) &inaddr)->sin6_addr = in6addr_any;
     addrlen = sizeof(struct sockaddr_in6);
@@ -510,19 +551,26 @@ static int create_listen6(void)
         port = htons(port);
 
         ((struct sockaddr_in6*) &inaddr)->sin6_port = port;
-        
+
         /* create a listen socket for incoming connections on this port */
         sd = socket(AF_INET6, SOCK_STREAM, 0);
         if (sd < 0) {
             if (EAFNOSUPPORT != opal_socket_errno) {
-                opal_output(0,"mca_oob_tcp_component_init: socket() failed: %s (%d)", 
+                opal_output(0,"mca_oob_tcp_component_init: socket() failed: %s (%d)",
                             strerror(opal_socket_errno), opal_socket_errno);
             }
             return ORTE_ERR_IN_ERRNO;
         }
-
-        /* setup socket options */
-        orte_oob_tcp_set_socket_options(sd);
+        /* Set the socket to close-on-exec so that no children inherit
+           this FD */
+        if (opal_fd_set_cloexec(sd) != OPAL_SUCCESS) {
+            opal_output(0, "mca_oob_tcp_create_listen6: unable to set the "
+                        "listening socket to CLOEXEC (%s:%d)\n",
+                        strerror(opal_socket_errno), opal_socket_errno);
+            CLOSE_THE_SOCKET(sd);
+            opal_argv_free(ports);
+            return ORTE_ERROR;
+        }
 
         /* Enable/disable reusing ports */
         if (orte_static_ports) {
@@ -538,7 +586,7 @@ static int create_listen6(void)
             opal_argv_free(ports);
             return ORTE_ERROR;
         }
-    
+
         if (bind(sd, (struct sockaddr*)&inaddr, addrlen) < 0) {
             if( (EADDRINUSE == opal_socket_errno) || (EADDRNOTAVAIL == opal_socket_errno) ) {
                 continue;
@@ -554,35 +602,37 @@ static int create_listen6(void)
         }
         /* resolve assigned port */
         if (getsockname(sd, (struct sockaddr*)&inaddr, &addrlen) < 0) {
-            opal_output(0, "mca_oob_tcp_create_listen: getsockname(): %s (%d)", 
+            opal_output(0, "mca_oob_tcp_create_listen: getsockname(): %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
             CLOSE_THE_SOCKET(sd);
             return ORTE_ERROR;
         }
-        
+
         /* setup listen backlog to maximum allowed by kernel */
         if (listen(sd, SOMAXCONN) < 0) {
-            opal_output(0, "mca_oob_tcp_component_init: listen(): %s (%d)", 
+            opal_output(0, "mca_oob_tcp_component_init: listen(): %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
             return ORTE_ERROR;
         }
-        
+
         /* set socket up to be non-blocking, otherwise accept could block */
         if ((flags = fcntl(sd, F_GETFL, 0)) < 0) {
-            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_GETFL) failed: %s (%d)", 
+            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_GETFL) failed: %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
             return ORTE_ERROR;
         }
         flags |= O_NONBLOCK;
         if (fcntl(sd, F_SETFL, flags) < 0) {
-            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_SETFL) failed: %s (%d)", 
+            opal_output(0, "mca_oob_tcp_component_init: fcntl(F_SETFL) failed: %s (%d)",
                         strerror(opal_socket_errno), opal_socket_errno);
             return ORTE_ERROR;
         }
 
         /* add this port to our connections */
         conn = OBJ_NEW(mca_oob_tcp_listener_t);
+        conn->tcp6 = true;
         conn->sd = sd;
+        conn->port = ntohs(((struct sockaddr_in6*) &inaddr)->sin6_port);
         opal_list_append(&mca_oob_tcp_component.listeners, &conn->item);
         /* and to our ports */
         asprintf(&tconn, "%d", ntohs(((struct sockaddr_in6*) &inaddr)->sin6_port));
@@ -608,7 +658,7 @@ static int create_listen6(void)
 
     /* done with this, so release it */
     opal_argv_free(ports);
-    
+
     return ORTE_SUCCESS;
 }
 #endif
@@ -641,15 +691,24 @@ static void* listen_thread(opal_object_t *obj)
             FD_SET(listener->sd, &readfds);
             max = (listener->sd > max) ? listener->sd : max;
         }
+        /* add the stop_thread fd */
+        FD_SET(mca_oob_tcp_component.stop_thread[0], &readfds);
+        max = (mca_oob_tcp_component.stop_thread[0] > max) ? mca_oob_tcp_component.stop_thread[0] : max;
+
         /* set timeout interval */
         timeout.tv_sec = mca_oob_tcp_component.listen_thread_tv.tv_sec;
         timeout.tv_usec = mca_oob_tcp_component.listen_thread_tv.tv_usec;
 
-        /* Block in a select for a short (10ms) amount of time to
-         * avoid hammering the cpu.  If a connection
+        /* Block in a select to avoid hammering the cpu.  If a connection
          * comes in, we'll get woken up right away.
          */
         rc = select(max + 1, &readfds, NULL, NULL, &timeout);
+        if (!mca_oob_tcp_component.listen_thread_active) {
+            /* we've been asked to terminate */
+            close(mca_oob_tcp_component.stop_thread[0]);
+            close(mca_oob_tcp_component.stop_thread[1]);
+            return NULL;
+        }
         if (rc < 0) {
             if (EAGAIN != opal_socket_errno && EINTR != opal_socket_errno) {
                 perror("select");
@@ -685,37 +744,83 @@ static void* listen_thread(opal_object_t *obj)
                  * OS might start rejecting connections due to timeout.
                  */
                 pending_connection = OBJ_NEW(mca_oob_tcp_pending_connection_t);
-                opal_event_set(orte_event_base, &pending_connection->ev, -1,
+                opal_event_set(orte_oob_base.ev_base, &pending_connection->ev, -1,
                                OPAL_EV_WRITE, connection_handler, pending_connection);
                 opal_event_set_priority(&pending_connection->ev, ORTE_MSG_PRI);
                 pending_connection->fd = accept(sd,
                                                 (struct sockaddr*)&(pending_connection->addr),
                                                 &addrlen);
+
+                /* check for < 0 as indicating an error upon accept */
                 if (pending_connection->fd < 0) {
-                    if (opal_socket_errno != EAGAIN || 
-                        opal_socket_errno != EWOULDBLOCK) {
-                        CLOSE_THE_SOCKET(pending_connection->fd);
-                        if (EMFILE == opal_socket_errno) {
-                            ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_SOCKETS);
-                            orte_show_help("help-orterun.txt", "orterun:sys-limit-sockets", true);
-                        } else {
-                            opal_output(0, "mca_oob_tcp_accept: accept() failed: %s (%d).",
-                                        strerror(opal_socket_errno), opal_socket_errno);
-                        }
-                        OBJ_RELEASE(pending_connection);
+                    OBJ_RELEASE(pending_connection);
+
+                    /* Non-fatal errors */
+                    if (EAGAIN == opal_socket_errno ||
+                        EWOULDBLOCK == opal_socket_errno) {
+                        continue;
+                    }
+
+                    /* If we run out of file descriptors, log an extra
+                       warning (so that the user can know to fix this
+                       problem) and abandon all hope. */
+                    else if (EMFILE == opal_socket_errno) {
+                        CLOSE_THE_SOCKET(sd);
+                        ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_SOCKETS);
+                        orte_show_help("help-oob-tcp.txt",
+                                       "accept failed",
+                                       true,
+                                       opal_process_info.nodename,
+                                       opal_socket_errno,
+                                       strerror(opal_socket_errno),
+                                       "Out of file descriptors");
                         goto done;
                     }
-                    continue;
+
+                    /* For all other cases, print a
+                       warning but try to continue */
+                    else {
+                        orte_show_help("help-oob-tcp.txt",
+                                       "accept failed",
+                                       true,
+                                       opal_process_info.nodename,
+                                       opal_socket_errno,
+                                       strerror(opal_socket_errno),
+                                       "Unknown cause; job will try to continue");
+                        continue;
+                    }
                 }
 
                 opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
-                                    "%s mca_oob_tcp_listen_thread: new connection: "
+                                    "%s mca_oob_tcp_listen_thread: incoming connection: "
                                     "(%d, %d) %s:%d\n",
                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                     pending_connection->fd, opal_socket_errno,
                                     opal_net_get_hostname((struct sockaddr*) &pending_connection->addr),
                                     opal_net_get_port((struct sockaddr*) &pending_connection->addr));
+
+                /* if we are on a privileged port, we only accept connections
+                 * from other privileged sockets. A privileged port is one
+                 * whose port is less than 1024 on Linux, so we'll check for that. */
+                if (1024 >= listener->port) {
+                    uint16_t inport;
+                    inport = opal_net_get_port((struct sockaddr*) &pending_connection->addr);
+                    if (1024 < inport) {
+                        /* someone tried to cross-connect privileges,
+                         * say something */
+                        orte_show_help("help-oob-tcp.txt",
+                                       "privilege failure", true,
+                                       opal_process_info.nodename, listener->port,
+                                       opal_net_get_hostname((struct sockaddr*) &pending_connection->addr),
+                                       inport);
+                        CLOSE_THE_SOCKET(pending_connection->fd);
+                        OBJ_RELEASE(pending_connection);
+                        continue;
+                    }
+                }
+
                 /* activate the event */
+                ORTE_POST_OBJECT(pending_connection);
                 opal_event_active(&pending_connection->ev, OPAL_EV_WRITE, 1);
                 accepted_connections++;
             }
@@ -755,14 +860,11 @@ static void* listen_thread(opal_object_t *obj)
 static void connection_handler(int sd, short flags, void* cbdata)
 {
     mca_oob_tcp_pending_connection_t *new_connection;
-    int i;
-    mca_oob_tcp_module_t *mod;
-    bool found;
-    struct sockaddr modaddr;
-    uint32_t netmask;
 
     new_connection = (mca_oob_tcp_pending_connection_t*)cbdata;
 
+    ORTE_ACQUIRE_OBJECT(new_connection);
+
     opal_output_verbose(4, orte_oob_base_framework.framework_output,
                         "%s connection_handler: working connection "
                         "(%d, %d) %s:%d\n",
@@ -771,41 +873,9 @@ static void connection_handler(int sd, short flags, void* cbdata)
                         opal_net_get_hostname((struct sockaddr*) &new_connection->addr),
                         opal_net_get_port((struct sockaddr*) &new_connection->addr));
 
-    /* cycle across all interfaces until we find the one that
-     * "owns" this connection - i.e., it is handling the
-     * incoming address space
-     */
-    found = false;
-    for(i = opal_ifbegin(); i >= 0; i = opal_ifnext(i)){
-        /* see if the incoming address is within the address space
-         * of this interface
-         */
-        if (OPAL_SUCCESS != opal_ifindextoaddr(i, &modaddr, sizeof(modaddr))) {
-            continue;
-        }
-        if (OPAL_SUCCESS != opal_ifindextomask(i, &netmask, sizeof(netmask))) {
-            continue;
-        }
-        if (opal_net_samenetwork((struct sockaddr*)&new_connection->addr, &modaddr, netmask)) {
-            /* lookup the corresponding kernel index of this interface */
-            i = opal_ifindextokindex(i);
-            /* get the module */
-            if (NULL == (mod = (mca_oob_tcp_module_t*)opal_pointer_array_get_item(&mca_oob_tcp_component.modules, i))) {
-                found = false;
-                break;
-            }
-            /* process the connection */
-            mod->api.accept_connection((struct mca_oob_tcp_module_t*)mod, new_connection->fd,
-                                       (struct sockaddr*) &(new_connection->addr));
-            found = true;
-            break;
-        }
-    }
-    if (!found) {
-        opal_output(0, "%s CONNECTION REQUEST ON UNKNOWN INTERFACE",
-                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-    }
-
+    /* process the connection */
+    mca_oob_tcp_module.accept_connection(new_connection->fd,
+                                         (struct sockaddr*) &(new_connection->addr));
     /* cleanup */
     OBJ_RELEASE(new_connection);
 }
@@ -818,11 +888,6 @@ static void connection_event_handler(int incoming_sd, short flags, void* cbdata)
     struct sockaddr addr;
     opal_socklen_t addrlen = sizeof(struct sockaddr);
     int sd;
-    int i;
-    mca_oob_tcp_module_t *mod;
-    bool found;
-    struct sockaddr modaddr;
-    uint32_t netmask;
 
     sd = accept(incoming_sd, (struct sockaddr*)&addr, &addrlen);
     opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
@@ -833,67 +898,56 @@ static void connection_event_handler(int incoming_sd, short flags, void* cbdata)
                         opal_net_get_hostname((struct sockaddr*) &addr),
                         opal_net_get_port((struct sockaddr*) &addr));
     if (sd < 0) {
-        if (EINTR == opal_socket_errno) {
+        /* Non-fatal errors */
+        if (EINTR == opal_socket_errno ||
+            EAGAIN == opal_socket_errno ||
+            EWOULDBLOCK == opal_socket_errno) {
             return;
         }
-        if (opal_socket_errno != EAGAIN && opal_socket_errno != EWOULDBLOCK) {
-            if (EMFILE == opal_socket_errno) {
-                /*
-                 * Close incoming_sd so that orte_show_help will have a file
-                 * descriptor with which to open the help file.  We will be
-                 * exiting anyway, so we don't need to keep it open.
-                 */
-                CLOSE_THE_SOCKET(incoming_sd);
-                ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_SOCKETS);
-                orte_show_help("help-orterun.txt", "orterun:sys-limit-sockets", true);
-            } else {
-                opal_output(0, "mca_oob_tcp_accept: accept() failed: %s (%d).", 
-                            strerror(opal_socket_errno), opal_socket_errno);
-            }
+
+        /* If we run out of file descriptors, log an extra warning (so
+           that the user can know to fix this problem) and abandon all
+           hope. */
+        else if (EMFILE == opal_socket_errno) {
+            CLOSE_THE_SOCKET(incoming_sd);
+            ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_SOCKETS);
+            orte_show_help("help-oob-tcp.txt",
+                           "accept failed",
+                           true,
+                           opal_process_info.nodename,
+                           opal_socket_errno,
+                           strerror(opal_socket_errno),
+                           "Out of file descriptors");
             orte_errmgr.abort(ORTE_ERROR_DEFAULT_EXIT_CODE, NULL);
+            return;
+        }
+
+        /* For all other cases, close the socket, print a warning but
+           try to continue */
+        else {
+            CLOSE_THE_SOCKET(incoming_sd);
+            orte_show_help("help-oob-tcp.txt",
+                           "accept failed",
+                           true,
+                           opal_process_info.nodename,
+                           opal_socket_errno,
+                           strerror(opal_socket_errno),
+                           "Unknown cause; job will try to continue");
+            return;
         }
-        return;
     }
 
-    /* cycle across all interfaces untile we find the one that
-     * "owns" this connection - i.e., it is handling the
-     * incoming address space
-     */
-    found = false;
-    for(i = opal_ifbegin(); i >= 0; i = opal_ifnext(i)){
-        /* see if the incoming address is within the address space
-         * of this interface
-         */
-        if (OPAL_SUCCESS != opal_ifindextoaddr(i, &modaddr, sizeof(modaddr))) {
-            continue;
-        }
-        if (OPAL_SUCCESS != opal_ifindextomask(i, &netmask, sizeof(netmask))) {
-            continue;
-        }
-        if (opal_net_samenetwork((struct sockaddr*)&addr, &modaddr, netmask)) {
-            /* lookup the corresponding kernel index of this interface */
-            i = opal_ifindextokindex(i);
-            /* get the module */
-            if (NULL == (mod = (mca_oob_tcp_module_t*)opal_pointer_array_get_item(&mca_oob_tcp_component.modules, i))) {
-                found = false;
-                break;
-            }
-            /* process the connection */
-            mod->api.accept_connection((struct mca_oob_tcp_module_t*)mod, sd, &addr);
-            found = true;
-            break;
-        }
-    }
-    if (!found) {
-        opal_output(0, "%s CONNECTION REQUEST ON UNKNOWN INTERFACE",
-                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-    }
+    /* process the connection */
+    mca_oob_tcp_module.accept_connection(sd, &addr);
 }
 
 
 static void tcp_ev_cons(mca_oob_tcp_listener_t* event)
 {
     event->ev_active = false;
+    event->tcp6 = false;
+    event->sd = -1;
+    event->port = 0;
 }
 static void tcp_ev_des(mca_oob_tcp_listener_t* event)
 {
@@ -901,6 +955,10 @@ static void tcp_ev_des(mca_oob_tcp_listener_t* event)
         opal_event_del(&event->event);
     }
     event->ev_active = false;
+    if (0 <= event->sd) {
+        CLOSE_THE_SOCKET(event->sd);
+        event->sd = -1;
+    }
 }
 
 OBJ_CLASS_INSTANCE(mca_oob_tcp_listener_t,
