@@ -9,10 +9,10 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
- * Copyright (c) 2014      Intel Corporation.  All rights reserved.
+ * Copyright (c) 2006-2018 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,18 +27,15 @@
  */
 
 #include "orte_config.h"
+#include "orte/runtime/orte_globals.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <signal.h>
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -52,6 +49,7 @@
 #include <fcntl.h>
 #endif
 
+#include "opal/mca/base/base.h"
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
@@ -63,11 +61,12 @@
 #include "orte/types.h"
 #include "orte/util/show_help.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/threads.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/runtime/orte_quit.h"
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/rmaps/rmaps.h"
+#include "orte/mca/rmaps/base/base.h"
 #include "orte/mca/state/state.h"
 
 #include "orte/orted/orted.h"
@@ -110,7 +109,6 @@ orte_plm_base_module_1_0_0_t orte_plm_slurm_module = {
  */
 static pid_t primary_srun_pid = 0;
 static bool primary_pid_set = false;
-static bool launching_daemons;
 static void launch_daemons(int fd, short args, void *cbdata);
 
 /**
@@ -119,12 +117,12 @@ static void launch_daemons(int fd, short args, void *cbdata);
 static int plm_slurm_init(void)
 {
     int rc;
-    
+
     if (ORTE_SUCCESS != (rc = orte_plm_base_comm_start())) {
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     /* if we don't want to launch (e.g., someone just wants
      * to test the mappers), then we assign vpids at "launch"
      * so the mapper has something to work with
@@ -157,7 +155,7 @@ static int plm_slurm_init(void)
  */
 static int plm_slurm_launch_job(orte_job_t *jdata)
 {
-    if (ORTE_JOB_CONTROL_RESTART & jdata->controls) {
+    if (ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_RESTART)) {
         /* this is a restart situation - skip to the mapping stage */
         ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_MAP);
     } else {
@@ -191,6 +189,8 @@ static void launch_daemons(int fd, short args, void *cbdata)
     orte_job_t *daemons;
     orte_state_caddy_t *state = (orte_state_caddy_t*)cbdata;
 
+    ORTE_ACQUIRE_OBJECT(state);
+
     OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                          "%s plm:slurm: LAUNCH DAEMONS CALLED",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
@@ -198,7 +198,7 @@ static void launch_daemons(int fd, short args, void *cbdata)
     /* if we are launching debugger daemons, then just go
      * do it - no new daemons will be launched
      */
-    if (ORTE_JOB_CONTROL_DEBUGGER_DAEMON & state->jdata->controls) {
+    if (ORTE_FLAG_TEST(state->jdata, ORTE_JOB_FLAG_DEBUGGER_DAEMON)) {
         state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
         ORTE_ACTIVATE_JOB_STATE(state->jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
         OBJ_RELEASE(state);
@@ -233,7 +233,7 @@ static void launch_daemons(int fd, short args, void *cbdata)
         rc = ORTE_ERR_NOT_FOUND;
         goto cleanup;
     }
-        
+
     if (0 == map->num_new_daemons) {
         /* set the state to indicate the daemons reported - this
          * will trigger the daemons_reported event and cause the
@@ -267,8 +267,10 @@ static void launch_daemons(int fd, short args, void *cbdata)
     /* start one orted on each node */
     opal_argv_append(&argc, &argv, "--ntasks-per-node=1");
 
-    /* alert us if any orteds die during startup */
-    opal_argv_append(&argc, &argv, "--kill-on-bad-exit");
+    if (!orte_enable_recovery) {
+        /* kill the job if any orteds die */
+        opal_argv_append(&argc, &argv, "--kill-on-bad-exit");
+    }
 
     /* ensure the orteds are not bound to a single processor,
      * just in case the TaskAffinity option is set by default.
@@ -278,6 +280,18 @@ static void launch_daemons(int fd, short args, void *cbdata)
      */
     opal_argv_append(&argc, &argv, "--cpu_bind=none");
 
+#if SLURM_CRAY_ENV
+    /*
+     * If in a SLURM/Cray env. make sure that Cray PMI is not pulled in,
+     * neither as a constructor run when orteds start, nor selected
+     * when pmix components are registered
+     */
+
+    opal_setenv("PMI_NO_PREINITIALIZE", "1", false, &orte_launch_environ);
+    opal_setenv("PMI_NO_FORK", "1", false, &orte_launch_environ);
+    opal_setenv("OMPI_NO_USE_CRAY_PMI", "1", false, &orte_launch_environ);
+#endif
+
     /* Append user defined arguments to srun */
     if ( NULL != mca_plm_slurm_component.custom_args ) {
         custom_strings = opal_argv_split(mca_plm_slurm_component.custom_args, ' ');
@@ -298,10 +312,10 @@ static void launch_daemons(int fd, short args, void *cbdata)
         /* if the daemon already exists on this node, then
          * don't include it
          */
-        if (node->daemon_launched) {
+        if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_DAEMON_LAUNCHED)) {
             continue;
         }
-        
+
         /* otherwise, add it to the list of nodes upon which
          * we need to launch a daemon
          */
@@ -336,19 +350,18 @@ static void launch_daemons(int fd, short args, void *cbdata)
     OPAL_OUTPUT_VERBOSE((2, orte_plm_base_framework.framework_output,
                          "%s plm:slurm: launching on nodes %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), nodelist_flat));
-    
+    free(nodelist_flat);
+
     /*
      * ORTED OPTIONS
      */
 
     /* add the daemon command (as specified by user) */
     orte_plm_base_setup_orted_cmd(&argc, &argv);
-    
+
     /* Add basic orted command line options, including debug flags */
     orte_plm_base_orted_append_basic_args(&argc, &argv,
-                                          NULL, &proc_vpid_index,
-                                          nodelist_flat);
-    free(nodelist_flat);
+                                          "slurm", &proc_vpid_index);
 
     /* tell the new daemons the base of the name list so they can compute
      * their own name on the other end
@@ -375,7 +388,8 @@ static void launch_daemons(int fd, short args, void *cbdata)
         if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(state->jdata->apps, n))) {
             continue;
         }
-        app_prefix_dir = app->prefix_dir;
+        app_prefix_dir = NULL;
+        orte_get_attribute(&app->attributes, ORTE_APP_PREFIX_DIR, (void**)&app_prefix_dir, OPAL_STRING);
         /* Check for already set cur_prefix_dir -- if different,
            complain */
         if (NULL != app_prefix_dir) {
@@ -396,27 +410,31 @@ static void launch_daemons(int fd, short args, void *cbdata)
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      cur_prefix));
             }
+            free(app_prefix_dir);
         }
     }
 
+    /* protect the args in case someone has a script wrapper around srun */
+    mca_base_cmd_line_wrap_args(argv);
+
     /* setup environment */
     env = opal_argv_copy(orte_launch_environ);
 
     if (0 < opal_output_get_verbosity(orte_plm_base_framework.framework_output)) {
         param = opal_argv_join(argv, ' ');
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
-                             "%s plm:slurm: final top-level argv:\n\t%s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             (NULL == param) ? "NULL" : param));
+        opal_output(orte_plm_base_framework.framework_output,
+                    "%s plm:slurm: final top-level argv:\n\t%s",
+                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                    (NULL == param) ? "NULL" : param);
         if (NULL != param) free(param);
     }
-    
+
     /* exec the daemon(s) */
     if (ORTE_SUCCESS != (rc = plm_slurm_start_proc(argc, argv, env, cur_prefix))) {
         ORTE_ERROR_LOG(rc);
         goto cleanup;
     }
-    
+
     /* indicate that the daemons for this job were launched */
     state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
     daemons->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
@@ -431,11 +449,11 @@ static void launch_daemons(int fd, short args, void *cbdata)
     if (NULL != env) {
         opal_argv_free(env);
     }
-    
+
     if(NULL != jobid_string) {
         free(jobid_string);
     }
-    
+
     /* cleanup the caddy */
     OBJ_RELEASE(state);
 
@@ -453,7 +471,7 @@ static int plm_slurm_terminate_orteds(void)
 {
     int rc=ORTE_SUCCESS;
     orte_job_t *jdata;
-        
+
     /* check to see if the primary pid is set. If not, this indicates
      * that we never launched any additional daemons, so we cannot
      * not wait for a waitpid to fire and tell us it's okay to
@@ -472,7 +490,7 @@ static int plm_slurm_terminate_orteds(void)
         jdata->num_terminated = jdata->num_procs;
         ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_DAEMONS_TERMINATED);
     }
-    
+
     return rc;
 }
 
@@ -496,25 +514,27 @@ static int plm_slurm_signal_job(orte_jobid_t jobid, int32_t signal)
 static int plm_slurm_finalize(void)
 {
     int rc;
-    
+
     /* cleanup any pending recvs */
     if (ORTE_SUCCESS != (rc = orte_plm_base_comm_stop())) {
         ORTE_ERROR_LOG(rc);
     }
-    
+
     return ORTE_SUCCESS;
 }
 
 
-static void srun_wait_cb(pid_t pid, int status, void* cbdata){
+static void srun_wait_cb(int sd, short fd, void *cbdata){
+    orte_wait_tracker_t *t2 = (orte_wait_tracker_t*)cbdata;
+    orte_proc_t *proc = t2->child;
     orte_job_t *jdata;
-    
+
     /* According to the SLURM folks, srun always returns the highest exit
      code of our remote processes. Thus, a non-zero exit status doesn't
      necessarily mean that srun failed - it could be that an orted returned
      a non-zero exit status. Of course, that means the orted failed(!), so
      the end result is the same - the job didn't start.
-     
+
      As a result, we really can't do much with the exit status itself - it
      could be something in errno (if srun itself failed), or it could be
      something returned by an orted, or it could be something returned by
@@ -523,36 +543,29 @@ static void srun_wait_cb(pid_t pid, int status, void* cbdata){
      now, though, the only thing that really matters is that
      srun failed. Report the error and make sure that orterun
      wakes up - otherwise, do nothing!
-     
+
      Unfortunately, the pid returned here is the srun pid, not the pid of
      the proc that actually died! So, to avoid confusion, just use -1 as the
      pid so nobody thinks this is real
      */
-    
+
     jdata = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid);
 
-    /* if we are in the launch phase, then any termination is bad */
-    if (launching_daemons) {
-        /* report that one or more daemons failed to launch so we can exit */
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
-                             "%s plm:slurm: daemon failed during launch",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_FAILED_TO_START);
-    } else {
-        /* if this is after launch, then we need to abort only if the status
-         * returned is non-zero - i.e., if the orteds exited with an error
+    /* abort only if the status returned is non-zero - i.e., if
+    * the orteds exited with an error
+     */
+    if (0 != proc->exit_code) {
+        /* an orted must have died unexpectedly - report
+         * that the daemon has failed so we exit
          */
-        if (0 != status) {
-            /* an orted must have died unexpectedly after launch - report
-             * that the daemon has failed so we exit
-             */
-            OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
-                                 "%s plm:slurm: daemon failed while running",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-            ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_ABORTED);
-        }
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
+                             "%s plm:slurm: srun returned non-zero exit status (%d) from launching the per-node daemon",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             proc->exit_code));
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_ABORTED);
+    } else {
         /* otherwise, check to see if this is the primary pid */
-        if (primary_srun_pid == pid) {
+        if (primary_srun_pid == proc->pid) {
             /* in this case, we just want to fire the proper trigger so
              * mpirun can exit
              */
@@ -564,6 +577,9 @@ static void srun_wait_cb(pid_t pid, int status, void* cbdata){
             ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_DAEMONS_TERMINATED);
         }
     }
+
+    /* done with this dummy */
+    OBJ_RELEASE(t2);
 }
 
 
@@ -573,9 +589,11 @@ static int plm_slurm_start_proc(int argc, char **argv, char **env,
     int fd;
     int srun_pid;
     char *exec_argv = opal_path_findv(argv[0], 0, env, NULL);
+    orte_proc_t *dummy;
 
     if (NULL == exec_argv) {
-        return ORTE_ERR_NOT_FOUND;
+        orte_show_help("help-plm-slurm.txt", "no-srun", true);
+        return ORTE_ERR_SILENT;
     }
 
     srun_pid = fork();
@@ -584,7 +602,22 @@ static int plm_slurm_start_proc(int argc, char **argv, char **env,
         free(exec_argv);
         return ORTE_ERR_SYS_LIMITS_CHILDREN;
     }
-    
+    /* if this is the primary launch - i.e., not a comm_spawn of a
+     * child job - then save the pid
+     */
+    if (0 < srun_pid && !primary_pid_set) {
+        primary_srun_pid = srun_pid;
+        primary_pid_set = true;
+    }
+
+    /* setup a dummy proc object to track the srun */
+    dummy = OBJ_NEW(orte_proc_t);
+    dummy->pid = srun_pid;
+    /* be sure to mark it as alive so we don't instantly fire */
+    ORTE_FLAG_SET(dummy, ORTE_PROC_FLAG_ALIVE);
+    /* setup the waitpid so we can find out if srun succeeds! */
+    orte_wait_cb(dummy, srun_wait_cb, orte_event_base, NULL);
+
     if (0 == srun_pid) {  /* child */
         char *bin_base = NULL, *lib_base = NULL;
 
@@ -630,25 +663,22 @@ static int plm_slurm_start_proc(int argc, char **argv, char **env,
             free(newenv);
         }
 
-        fd = open("/dev/null", O_CREAT|O_WRONLY|O_TRUNC, 0666);
-        if(fd > 0) {
+        fd = open("/dev/null", O_CREAT|O_RDWR|O_TRUNC, 0666);
+        if (fd >= 0) {
             dup2(fd, 0);
-        }
+            /* When not in debug mode and --debug-daemons was not passed,
+             * tie stdout/stderr to dev null so we don't see messages from orted
+             * EXCEPT if the user has requested that we leave sessions attached
+             */
+            if (0 > opal_output_get_verbosity(orte_plm_base_framework.framework_output) &&
+                !orte_debug_daemons_flag && !orte_leave_session_attached) {
+                dup2(fd,1);
+                dup2(fd,2);
+            }
 
-        /* When not in debug mode and --debug-daemons was not passed,
-         * tie stdout/stderr to dev null so we don't see messages from orted
-         * EXCEPT if the user has requested that we leave sessions attached
-         */
-        if (0 >= opal_output_get_verbosity(orte_plm_base_framework.framework_output) &&
-            !orte_debug_daemons_flag && !orte_leave_session_attached) {
-            if (fd >= 0) {
-                if (fd != 1) {
-                    dup2(fd,1);
-                } else if (fd != 2) {
-                    dup2(fd,2);
-                } else {
-                    close(fd);
-                }
+            /* Don't leave the extra fd to /dev/null open */
+            if (fd > 2) {
+                close(fd);
             }
         }
 
@@ -665,20 +695,10 @@ static int plm_slurm_start_proc(int argc, char **argv, char **env,
         exit(1);
     } else {  /* parent */
         /* just in case, make sure that the srun process is not in our
-        process group any more.  Stevens says always do this on both
-        sides of the fork... */
+           process group any more.  Stevens says always do this on both
+           sides of the fork... */
         setpgid(srun_pid, srun_pid);
-        
-        /* if this is the primary launch - i.e., not a comm_spawn of a
-         * child job - then save the pid
-         */
-        if (!primary_pid_set) {
-            primary_srun_pid = srun_pid;
-            primary_pid_set = true;
-        }
-        
-        /* setup the waitpid so we can find out if srun succeeds! */
-        orte_wait_cb(srun_pid, srun_wait_cb, NULL);
+
         free(exec_argv);
     }
 
