@@ -1,10 +1,10 @@
 /*
- * Copyright (c) 2013      Mellanox Technologies, Inc.
+ * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "oshmem_config.h"
@@ -19,15 +19,38 @@
 #include "oshmem/shmem/shmem_api_logger.h"
 #include "oshmem/mca/memheap/memheap.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/include/pshmem.h"
+#pragma weak shmem_realloc = pshmem_realloc
+#pragma weak shrealloc = pshrealloc
+#include "oshmem/shmem/c/profile/defines.h"
+#endif
+
+static inline void* _shrealloc(void *ptr, size_t size);
+
+void* shmem_realloc(void *ptr, size_t size)
+{
+    return _shrealloc(ptr, size);
+}
+
 void* shrealloc(void *ptr, size_t size)
+{
+    return _shrealloc(ptr, size);
+}
+
+static inline void* _shrealloc(void *ptr, size_t size)
 {
     int rc;
     void* pBuff = NULL;
 
     RUNTIME_CHECK_INIT();
 
+    SHMEM_MUTEX_LOCK(shmem_internal_mutex_alloc);
+
     rc = MCA_MEMHEAP_CALL(realloc(size, ptr, &pBuff));
 
+    SHMEM_MUTEX_UNLOCK(shmem_internal_mutex_alloc);
+
     if (OSHMEM_SUCCESS != rc) {
         SHMEM_API_VERBOSE(1,
                           "Allocation with shrealloc(ptr=%p, size=%lu) failed.",
